"""Config parser module."""

from __future__ import annotations

import contextlib
import dataclasses
import os
import pathlib

from types import MappingProxyType
from typing import (
    Any,
    cast,
    Dict,
    get_type_hints,
    List,
    Optional,
    Protocol,
    runtime_checkable,
    Tuple,
    Type,
    TYPE_CHECKING,
    Union,
)

import tomli
import tomli_w
import yaml

from tm_devices.helpers import (
    AliasDict,
    ConfigFileType,
    ConnectionTypes,
    DeviceConfigEntry,
    DeviceTypes,
    DMConfigOptions,
    SerialConfig,
)
from tm_devices.helpers.constants_and_dataclasses import (
    CONFIG_CLASS_STR_PREFIX_MAPPING,
)

if TYPE_CHECKING:
    from collections.abc import Mapping


@runtime_checkable
@dataclasses.dataclass
class _DataclassProtocol(Protocol):
    """A Protocol class to allow for type hinting things that accept generic dataclasses."""


class DMConfigParser:
    """Class to parse a configuration file for device connections and runtime options."""

    DEVICES_ENV_VARIABLE = "TM_DEVICES"
    """The name of the environment variable which can contain a list of device configurations.

    Multiple devices are separated by "~~~", each device contains a comma-separated list of device
    parameters to use when connecting to the device. See the [configuration
    documentation][environment-variable] for the device configuration environment variable syntax.
    """
    OPTIONS_ENV_VARIABLE = "TM_OPTIONS"
    """The name of the environment variable which can contain a comma-separated list of options.

    See the [configuration documentation][environment-variable] for the options environment variable
    syntax.
    """
    CONFIG_FILE_PATH_ENV_VARIABLE = "TM_DEVICES_CONFIG"
    """The name of the environment variable to check for a custom config file path.

    This environment variable is checked to see if there is a user-provided path to a custom config
    file that contains a list of devices and hostnames/IP addresses.
    """
    DEFAULT_CONFIG_FILE_PATH = "./tm_devices.yaml"
    """The default location for the config file.

    If present, the file should contain a list of devices and hostnames/IP addresses. See the
    [configuration documentation][config-file] for the config file syntax.
    """
    FileType = ConfigFileType
    """A convenience enumeration listing the valid config file types."""

    _CONFIG_NESTED_DICT_MAPPING: Mapping[
        Union[Type[_DataclassProtocol], Type[SerialConfig]], str
    ] = MappingProxyType({SerialConfig: "serial_config"})

    ################################################################################################
    # Magic Methods
    ################################################################################################
    def __init__(self) -> None:
        """Initialize the DMConfigParser and read the configuration."""
        # Create a count of the number of each type of device created
        self.__dev_count: Dict[str, int] = dict.fromkeys(DeviceTypes.list_values(), 0)

        self.__devices: Dict[str, DeviceConfigEntry] = AliasDict()
        self.__options = DMConfigOptions()

        # Check if the environment variables exists
        if self.DEVICES_ENV_VARIABLE in os.environ or self.OPTIONS_ENV_VARIABLE in os.environ:
            devices_list = self.__parse_env_devices()
            self.update_options(self.__parse_env_options())

            self.__pre_process_env_devices(devices_list)
            self.__add_from_device_list(devices_list)
        else:
            # Errors can be ignored, no default file was found
            with contextlib.suppress(FileNotFoundError):
                self.load_config_file()

    def __str__(self) -> str:
        """Return string representation of instance as environment variables."""
        return (
            f"{self.OPTIONS_ENV_VARIABLE}={self.__options}\n"
            f"{self.DEVICES_ENV_VARIABLE}="
            f"{'~~~'.join(['', *map(str, self.__devices.values()), ''])}"
        )

    ################################################################################################
    # Properties
    ################################################################################################
    @property
    def defined_config_file_path(self) -> str:
        """Filepath of the config file.

        Prioritizes the path contained in the environment variable defined by
        [`DMConfigParser.CONFIG_FILE_PATH_ENV_VARIABLE`][tm_devices.components.DMConfigParser.CONFIG_FILE_PATH_ENV_VARIABLE]
        with a fallback to the default path defined by
        [`DMConfigParser.DEFAULT_CONFIG_FILE_PATH`][tm_devices.components.DMConfigParser.DEFAULT_CONFIG_FILE_PATH].
        """
        return os.getenv(
            self.CONFIG_FILE_PATH_ENV_VARIABLE,
            self.DEFAULT_CONFIG_FILE_PATH,
        )

    @property
    def devices(self) -> Mapping[str, DeviceConfigEntry]:
        """Return the device dict as a read-only object."""
        return MappingProxyType(self.__devices)

    @property
    def options(self) -> DMConfigOptions:
        """Return the config options."""
        return self.__options

    ################################################################################################
    # Public Methods
    ################################################################################################
    def add_device(  # noqa: PLR0913  # pylint: disable=too-many-locals
        self,
        *,
        device_type: Union[DeviceTypes, str],
        address: str,
        connection_type: Union[ConnectionTypes, str] = ConnectionTypes.TCPIP,
        alias: Optional[str] = None,
        lan_port: Optional[int] = None,
        lan_device_name: Optional[str] = None,
        serial_config: Optional[SerialConfig] = None,
        device_driver: Optional[str] = None,
        gpib_board_number: Optional[int] = None,
    ) -> Tuple[str, DeviceConfigEntry]:
        """Add a new device configuration entry.

        Args:
            device_type: The specific device type defined in the config entry.
            address: The address used to connect to the device:
                TCPIP: IP address or the hostname.
                SOCKET: IP address or the hostname (must define `lan_port` field).
                REST_API: IP address or the hostname (must define `lan_port` field).
                SERIAL/ASRL: serial COM port number.
                USB: use expression format f"{model}-{serial_number}" (ex: "MSO24-ABC0123").
            connection_type: The specific type of connection defined in the config entry.
            alias: An optional key/name used to retrieve this device from the DeviceManager.
            lan_port: The port number to connect on, used for SOCKET/REST_API connections.
            lan_device_name: The LAN device name to connect on, only used for TCPIP connections.
                The default is 'inst0'.
            serial_config: A dataclass for holding serial connection info.
            device_driver: A string indicating the specific Python device driver to use.
            gpib_board_number: The GPIB board number (also referred to as a controller), only used
                for GPIB connections. The default is 0.

        Returns:
            Tuple of the config entry name and the new DeviceConfigEntry

        Raises:
            ValueError: invalid config options, trouble creating (or a duplicate) device entry.
        """
        device_type = cast("DeviceTypes", device_type)
        connection_type = cast("ConnectionTypes", connection_type)
        # device and connection type conversion to enum from string is handled on initialization
        new_entry = DeviceConfigEntry(
            device_type=device_type,
            address=address,
            connection_type=connection_type,
            alias=alias,
            lan_port=lan_port,
            lan_device_name=lan_device_name,
            serial_config=serial_config,
            device_driver=device_driver,
            gpib_board_number=gpib_board_number,
        )
        # Currently, USB connections when using the PyVISA-py backend are not supported.
        if self.__options.standalone and new_entry.connection_type in {
            ConnectionTypes.USB,
            ConnectionTypes.GPIB,
        }:
            message = (
                f"{new_entry.connection_type.value} connections using PyVISA-py (standalone) "
                f"are not supported.\n{new_entry}"
            )
            raise ValueError(message)

        # Validate the connection is unique
        for dev_entry in self.__devices.values():
            new_entry_potential_hostname = new_entry.address.split(".")[0]
            dev_entry_potential_hostname = dev_entry.address.split(".")[0]
            if (new_entry.get_address_expression() == dev_entry.get_address_expression()) or (
                new_entry_potential_hostname == dev_entry_potential_hostname
                and not (
                    new_entry_potential_hostname.isdigit() or dev_entry_potential_hostname.isdigit()
                )
            ):
                message = (
                    f"Found duplicate addresses in the "
                    f'configuration for "{new_entry.get_address_expression()}":'
                    f"\n\t    Entry 1: {dev_entry}\n\tNew Entry 2: {new_entry}"
                )
                raise ValueError(message)

        # increment count
        self.__dev_count[new_entry.device_type.value] += 1
        device_number = self.__dev_count[new_entry.device_type.value]
        new_entry_name = f"{new_entry.device_type.value} {device_number}"
        # store the new entry
        self.__devices[new_entry_name] = new_entry
        if new_entry.alias:
            self.__devices[new_entry.alias] = new_entry
        return new_entry_name, new_entry

    def load_config_file(
        self, config_file_path: Optional[Union[str, os.PathLike[str]]] = None
    ) -> None:
        """Load in the config file located at the given path.

        This method will update any changed options and add any newly defined devices.

        Args:
            config_file_path: The path to the config file to load.
        """
        # Grabs path from environment variable with fallback to the default path
        file_path = config_file_path if config_file_path else self.defined_config_file_path
        # Automatically handles toml and yaml
        options, devices_list = self.__parse_config_file(file_path)
        self.__add_from_device_list(devices_list)
        self.update_options(options)

    def remove_device(self, device_name: str) -> None:
        """Remove a device from the configuration.

        Args:
            device_name: The name of the device to remove from the internal dictionary.
        """
        del self.__devices[device_name]

    def to_config_file_text(self, file_type: ConfigFileType = ConfigFileType.YAML) -> str:
        """Return string representation of instance as config file contents.

        Args:
            file_type: The type of file to create, yaml or toml.
        """
        # order of dict matters to match auto-formatted toml and yaml
        if file_type == self.FileType.TOML:
            retval = tomli_w.dumps(
                {
                    "devices": [dev.to_dict(ignore_none=True) for dev in self.devices.values()],
                    "options": self.options.to_dict(ignore_none=True),
                }
            )
        else:
            retval = yaml.dump(
                {
                    "options": self.options.to_dict(ignore_none=True),
                    "devices": [dev.to_dict(ignore_none=True) for dev in self.devices.values()],
                }
            )
            # add document header
            retval = "---\n" + retval
            # add extra spacing under devices section
            devs_idx = retval.find("devices:") + 8
            opts_idx = retval.find("options:") - 1
            retval = retval.replace(
                retval[devs_idx:opts_idx], retval[devs_idx:opts_idx].replace("\n", "\n  ")
            )
        return retval

    def update_options(self, new_options: DMConfigOptions) -> None:
        """Update the current configuration options.

        This ensures that any time new options are loaded in only explicitly defined options will be
        updated.

        Args:
            new_options: The new options to merge with the current options.
        """
        self.__options = dataclasses.replace(
            self.__options,
            **new_options.to_dict(ignore_none=True),
        )

    def write_config_to_file(
        self, config_file_path: Optional[Union[str, os.PathLike[str]]] = None
    ) -> str:
        """Write the current configuration to a config file.

        This method will overwrite any existing config file with the current devices and options. If
        no custom path is provided, it will write to the file defined by
        [`DMConfigParser.defined_config_file_path`][tm_devices.components.DMConfigParser.defined_config_file_path].

        Args:
            config_file_path: The path to the config file. If ends in ".toml" will create toml file.

        Returns:
            Path to the written file.
        """
        if config_file_path is None:
            config_file_path = self.defined_config_file_path

        config_path_obj = pathlib.Path(config_file_path)

        file_type = (
            DMConfigParser.FileType.TOML
            if config_path_obj.as_posix().lower().endswith(".toml")
            else DMConfigParser.FileType.YAML
        )
        config_path_obj.write_text(self.to_config_file_text(file_type=file_type), encoding="utf-8")

        return config_path_obj.as_posix()

    ################################################################################################
    # Private Methods
    ################################################################################################
    def __add_from_device_list(self, devices_list: List[Dict[str, Any]]) -> None:
        """Add devices from a list to the current config.

        Args:
            devices_list: The list of devices to add to the current config.
        """
        for entry in devices_list:
            # Instantiate nested classes from fields that store kwargs-style dictionaries
            for to_class, entry_key in self._CONFIG_NESTED_DICT_MAPPING.items():
                if config_dict := entry.get(entry_key):
                    # noinspection PyArgumentList
                    entry[entry_key] = to_class(**config_dict)
            self.add_device(**entry)

    def __parse_env_options(self) -> DMConfigOptions:
        """Extract the options flags from the env variable TM_OPTIONS.

        Returns:
            Dataclass of config options/flags.

        Raises:
            KeyError: Indicates unrecognized option name.
        """
        options_dict = {
            arg.strip().split("=", maxsplit=1)[0]: bool(arg)
            if "=" not in arg
            else arg.split("=", maxsplit=1)[-1]
            for arg in os.getenv(self.OPTIONS_ENV_VARIABLE, "").split(",")
            if arg
        }
        valid_config_options = {option.upper() for option in get_type_hints(DMConfigOptions)}
        if valid_config_options.issuperset(options_dict):
            options = {
                arg_name.lower(): int(options_dict[arg_name])
                if isinstance(options_dict[arg_name], str) and options_dict[arg_name].isdigit()  # pyright: ignore[reportUnknownMemberType,reportAttributeAccessIssue]
                else options_dict[arg_name]
                for arg_name in options_dict
            }
            return DMConfigOptions(**options)  # pyright: ignore[reportArgumentType]
        msg = (
            f"Invalid configuration options found: {list(set(options_dict) - valid_config_options)}"
        )
        raise KeyError(msg)

    def __parse_env_devices(self) -> List[Dict[str, Any]]:
        """Extract list of device configuration dictionaries from the env variable TM_DEVICES.

        Returns:
            List of device entry dictionaries.

        Raises:
            ValueError: invalid string formatting cannot be parsed.
        """
        devices_str_list = [
            arg.strip() for arg in os.getenv(self.DEVICES_ENV_VARIABLE, "").split("~~~") if arg
        ]
        retval: List[Dict[str, Any]] = []
        msg: List[str] = []
        for dev_entry in devices_str_list:
            temp_dict: Dict[str, Any] = {}
            for dev_arg in dev_entry.split(","):
                try:
                    dev_key, dev_val = dev_arg.split("=", 1)
                    temp_dict[dev_key] = dev_val
                except ValueError:  # noqa: PERF203
                    msg.append(
                        f'Expected device defined like "device_type=SCOPE,address=1.2.3.4,...",\n'
                        f'cannot parse "{dev_entry}"'
                    )
            retval.append(temp_dict)
        if msg:
            raise ValueError("\n".join(msg))
        return retval

    @staticmethod
    def __parse_config_file(
        config_file_path: Union[str, os.PathLike[str]],
    ) -> Tuple[DMConfigOptions, List[Dict[str, Any]]]:
        """Parse config file for the options flags and list of device configuration dictionaries.

        Args:
            config_file_path: The path to the config file to load.

        Returns:
            Tuple with dataclass of config options/flags and list of device entry dictionaries.

        Raises:
            FileNotFoundError: if the defined config file does not exist.
            KeyError: Indicates unrecognized option name.
        """
        config_path = pathlib.Path(config_file_path)  # normalize the path
        if not config_path.is_file():
            raise FileNotFoundError(config_path)
        # read in data
        with config_path.open(encoding="utf-8") as config_file:
            if config_path.suffix == ".toml":
                data = tomli.loads(config_file.read())
            else:  # ["yaml", "yml"]
                data = yaml.safe_load(config_file)
        # return data with some pre-processing
        options_mapping = data.get("options", {})
        try:
            options = DMConfigOptions(**options_mapping)
        except TypeError:
            options_list = list(options_mapping)
            valid_options = set(get_type_hints(DMConfigOptions))
            msg = f"Invalid configuration options found: {list(set(options_list) - valid_options)}"
            raise KeyError(msg)  # noqa: B904
        devices_list = data.get("devices", [])
        return options, devices_list

    def __pre_process_env_devices(self, devices_list: List[Dict[str, Any]]) -> None:
        """Modify device entries in place to make env data mimic toml/yaml parser output format."""
        for entry in devices_list:
            # Bundle any prefixed pairs into a nested kwarg-style dict.
            for to_class, prefix in CONFIG_CLASS_STR_PREFIX_MAPPING.items():
                config_dict: Dict[str, str] = {}
                if config_dict := {
                    key.replace(prefix, ""): entry.pop(key)
                    for key in list(entry.keys())
                    if key.startswith(prefix)
                    and (key not in self._CONFIG_NESTED_DICT_MAPPING.values())
                }:
                    # assign the dict to the correct key via the class the prefix represented
                    entry[self._CONFIG_NESTED_DICT_MAPPING[to_class]] = config_dict



"""The measu commands module.

These commands are used in the following models:
TekScopePC

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Commands and Queries:
    ```
    - MEASU:MEAS1:SUBGROUP:RESUlts:ALLAcqs:MAXimum? <Qstring>
    - MEASU:MEAS1:SUBGROUP:RESUlts:ALLAcqs:MEAN? <Qstring>
    - MEASU:MEAS1:SUBGROUP:RESUlts:ALLAcqs:MINimum? <Qstring>
    - MEASU:MEAS1:SUBGROUP:RESUlts:ALLAcqs:PK2PK? <Qstring>
    - MEASU:MEAS1:SUBGROUP:RESUlts:ALLAcqs:POPUlation? <Qstring>
    - MEASU:MEAS1:SUBGROUP:RESUlts:ALLAcqs:STDDev? <Qstring>
    - MEASU:MEAS1:SUBGROUP:RESUlts:CURRentacq:MAXimum? <Qstring>
    - MEASU:MEAS1:SUBGROUP:RESUlts:CURRentacq:MEAN? <Qstring>
    - MEASU:MEAS1:SUBGROUP:RESUlts:CURRentacq:MINimum? <Qstring>
    - MEASU:MEAS1:SUBGROUP:RESUlts:CURRentacq:PK2PK? <Qstring>
    - MEASU:MEAS1:SUBGROUP:RESUlts:CURRentacq:POPUlation? <Qstring>
    - MEASU:MEAS1:SUBGROUP:RESUlts:CURRentacq:STDDev? <Qstring>
    ```
"""

from typing import Optional, TYPE_CHECKING

from ..helpers import SCPICmdRead, SCPICmdReadWithArguments

if TYPE_CHECKING:
    from tm_devices.driver_mixins.device_control.pi_control import PIControl


class MeasuMeas1SubgroupResultsCurrentacqStddev(SCPICmdReadWithArguments):
    r"""The ``MEASU:MEAS1:SUBGROUP:RESUlts:CURRentacq:STDDev`` command.

    Description:
        - This query returns the standard deviation value of the measurement specified by the
          string, for the current acquisition.

    Usage:
        - Using the ``.query(argument)`` method will send the
          ``MEASU:MEAS1:SUBGROUP:RESUlts:CURRentacq:STDDev? argument`` query.
        - Using the ``.verify(argument, value)`` method will send the
          ``MEASU:MEAS1:SUBGROUP:RESUlts:CURRentacq:STDDev? argument`` query and raise an
          AssertionError if the returned value does not match ``value``.

    SCPI Syntax:
        ```
        - MEASU:MEAS1:SUBGROUP:RESUlts:CURRentacq:STDDev? <Qstring>
        ```

    Info:
        - ``<Qstring>`` = INPUT\|OUTPUT1\|OUTPUT2\|OUTPUT3\|OUTPUT4\|OUTPUT5\| OUTPUT6\|OUTPUT7\|
          RAIL1DPMOVERSHOOT\|RAIL1DPMUNDERSHOOT \|RAIL1FREQUENCY \|RAIL1PK2PK \|RAIL1RMS
          \|RAIL1RMSFULL\|RAIL2FREQUENCY \|RAIL2PK2PK \|RAIL2RMS \|RAIL2RMSFULL \|RAIL3FREQUENCY
          \|RAIL3PK2PK \|RAIL3RMS\|RAIL3RMSFULL \|RAIL4FREQUENCY \|RAIL4PK2PK \|RAIL4RMS
          \|RAIL4RMSFULL \|RAIL5FREQUENCY \|RAIL5PK2PK\|RAIL5RMS \|RAIL5RMSFULL \|RAIL6FREQUENCY
          \|RAIL6PK2PK \|RAIL6RMS \|RAIL6RMSFULL \|RAIL7DPMOVERSHOOT \|RAIL7DPMUNDERSHOOT
          \|RAIL7FREQUENCY \|RAIL7PK2PK \|RAIL7RMS \|RAIL7RMSFULL.
    """


class MeasuMeas1SubgroupResultsCurrentacqPopulation(SCPICmdReadWithArguments):
    r"""The ``MEASU:MEAS1:SUBGROUP:RESUlts:CURRentacq:POPUlation`` command.

    Description:
        - This query returns the population value of the measurement specified by the string, for
          the current acquisition.

    Usage:
        - Using the ``.query(argument)`` method will send the
          ``MEASU:MEAS1:SUBGROUP:RESUlts:CURRentacq:POPUlation? argument`` query.
        - Using the ``.verify(argument, value)`` method will send the
          ``MEASU:MEAS1:SUBGROUP:RESUlts:CURRentacq:POPUlation? argument`` query and raise an
          AssertionError if the returned value does not match ``value``.

    SCPI Syntax:
        ```
        - MEASU:MEAS1:SUBGROUP:RESUlts:CURRentacq:POPUlation? <Qstring>
        ```

    Info:
        - ``<Qstring>`` = INPUT\|OUTPUT1\|OUTPUT2\|OUTPUT3\|OUTPUT4\|OUTPUT5\| OUTPUT6\|OUTPUT7\|
          RAIL1DPMOVERSHOOT\|RAIL1DPMUNDERSHOOT \|RAIL1FREQUENCY \|RAIL1PK2PK \|RAIL1RMS
          \|RAIL1RMSFULL\|RAIL2FREQUENCY \|RAIL2PK2PK \|RAIL2RMS \|RAIL2RMSFULL \|RAIL3FREQUENCY
          \|RAIL3PK2PK \|RAIL3RMS\|RAIL3RMSFULL \|RAIL4FREQUENCY \|RAIL4PK2PK \|RAIL4RMS
          \|RAIL4RMSFULL \|RAIL5FREQUENCY \|RAIL5PK2PK\|RAIL5RMS \|RAIL5RMSFULL \|RAIL6FREQUENCY
          \|RAIL6PK2PK \|RAIL6RMS \|RAIL6RMSFULL \|RAIL7DPMOVERSHOOT \|RAIL7DPMUNDERSHOOT
          \|RAIL7FREQUENCY \|RAIL7PK2PK \|RAIL7RMS \|RAIL7RMSFULL.
    """


class MeasuMeas1SubgroupResultsCurrentacqPk2pk(SCPICmdReadWithArguments):
    r"""The ``MEASU:MEAS1:SUBGROUP:RESUlts:CURRentacq:PK2PK`` command.

    Description:
        - This query returns the peak-to-peak value of the measurement specified by the string, for
          the current acquisition.

    Usage:
        - Using the ``.query(argument)`` method will send the
          ``MEASU:MEAS1:SUBGROUP:RESUlts:CURRentacq:PK2PK? argument`` query.
        - Using the ``.verify(argument, value)`` method will send the
          ``MEASU:MEAS1:SUBGROUP:RESUlts:CURRentacq:PK2PK? argument`` query and raise an
          AssertionError if the returned value does not match ``value``.

    SCPI Syntax:
        ```
        - MEASU:MEAS1:SUBGROUP:RESUlts:CURRentacq:PK2PK? <Qstring>
        ```

    Info:
        - ``<Qstring>`` = INPUT\|OUTPUT1\|OUTPUT2\|OUTPUT3\|OUTPUT4\|OUTPUT5\| OUTPUT6\|OUTPUT7\|
          RAIL1DPMOVERSHOOT\|RAIL1DPMUNDERSHOOT \|RAIL1FREQUENCY \|RAIL1PK2PK \|RAIL1RMS
          \|RAIL1RMSFULL\|RAIL2FREQUENCY \|RAIL2PK2PK \|RAIL2RMS \|RAIL2RMSFULL \|RAIL3FREQUENCY
          \|RAIL3PK2PK \|RAIL3RMS\|RAIL3RMSFULL \|RAIL4FREQUENCY \|RAIL4PK2PK \|RAIL4RMS
          \|RAIL4RMSFULL \|RAIL5FREQUENCY \|RAIL5PK2PK\|RAIL5RMS \|RAIL5RMSFULL \|RAIL6FREQUENCY
          \|RAIL6PK2PK \|RAIL6RMS \|RAIL6RMSFULL \|RAIL7DPMOVERSHOOT \|RAIL7DPMUNDERSHOOT
          \|RAIL7FREQUENCY \|RAIL7PK2PK \|RAIL7RMS \|RAIL7RMSFULL.
    """


class MeasuMeas1SubgroupResultsCurrentacqMinimum(SCPICmdReadWithArguments):
    r"""The ``MEASU:MEAS1:SUBGROUP:RESUlts:CURRentacq:MINimum`` command.

    Description:
        - This query returns the minimum value of the measurement specified by the string, for the
          current acquisition.

    Usage:
        - Using the ``.query(argument)`` method will send the
          ``MEASU:MEAS1:SUBGROUP:RESUlts:CURRentacq:MINimum? argument`` query.
        - Using the ``.verify(argument, value)`` method will send the
          ``MEASU:MEAS1:SUBGROUP:RESUlts:CURRentacq:MINimum? argument`` query and raise an
          AssertionError if the returned value does not match ``value``.

    SCPI Syntax:
        ```
        - MEASU:MEAS1:SUBGROUP:RESUlts:CURRentacq:MINimum? <Qstring>
        ```

    Info:
        - ``<Qstring>`` = INPUT\|OUTPUT1\|OUTPUT2\|OUTPUT3\|OUTPUT4\|OUTPUT5\| OUTPUT6\|OUTPUT7\|
          RAIL1DPMOVERSHOOT\|RAIL1DPMUNDERSHOOT \|RAIL1FREQUENCY \|RAIL1PK2PK \|RAIL1RMS
          \|RAIL1RMSFULL\|RAIL2FREQUENCY \|RAIL2PK2PK \|RAIL2RMS \|RAIL2RMSFULL \|RAIL3FREQUENCY
          \|RAIL3PK2PK \|RAIL3RMS\|RAIL3RMSFULL \|RAIL4FREQUENCY \|RAIL4PK2PK \|RAIL4RMS
          \|RAIL4RMSFULL \|RAIL5FREQUENCY \|RAIL5PK2PK\|RAIL5RMS \|RAIL5RMSFULL \|RAIL6FREQUENCY
          \|RAIL6PK2PK \|RAIL6RMS \|RAIL6RMSFULL \|RAIL7DPMOVERSHOOT \|RAIL7DPMUNDERSHOOT
          \|RAIL7FREQUENCY \|RAIL7PK2PK \|RAIL7RMS \|RAIL7RMSFULL.
    """


class MeasuMeas1SubgroupResultsCurrentacqMean(SCPICmdReadWithArguments):
    r"""The ``MEASU:MEAS1:SUBGROUP:RESUlts:CURRentacq:MEAN`` command.

    Description:
        - This query returns the mean value of the measurement specified by the string, for the
          current acquisition.

    Usage:
        - Using the ``.query(argument)`` method will send the
          ``MEASU:MEAS1:SUBGROUP:RESUlts:CURRentacq:MEAN? argument`` query.
        - Using the ``.verify(argument, value)`` method will send the
          ``MEASU:MEAS1:SUBGROUP:RESUlts:CURRentacq:MEAN? argument`` query and raise an
          AssertionError if the returned value does not match ``value``.

    SCPI Syntax:
        ```
        - MEASU:MEAS1:SUBGROUP:RESUlts:CURRentacq:MEAN? <Qstring>
        ```

    Info:
        - ``<Qstring>`` = INPUT \|OUTPUT1 \|OUTPUT2 \|OUTPUT3 \|OUTPUT4 \|OUTPUT5 \|OUTPUT6
          \|OUTPUT7 \|RAIL1DPMOVERSHOOT \|RAIL1DPMUNDERSHOOT \|RAIL1FREQUENCY \|RAIL1PK2PK
          \|RAIL1RMS \|RAIL1RMSFULL \|RAIL2FREQUENCY \|RAIL2PK2PK \|RAIL2RMS \|RAIL2RMSFULL
          \|RAIL3FREQUENCY \|RAIL3PK2PK \|RAIL3RMS \|RAIL3RMSFULL \|RAIL4FREQUENCY \|RAIL4PK2PK
          \|RAIL4RMS \|RAIL4RMSFULL \|RAIL5FREQUENCY \|RAIL5PK2PK \|RAIL5RMS \|RAIL5RMSFULL
          \|RAIL6FREQUENCY \|RAIL6PK2PK \|RAIL6RMS \|RAIL6RMSFULL \|RAIL7DPMOVERSHOOT
          \|RAIL7DPMUNDERSHOOT \|RAIL7FREQUENCY \|RAIL7PK2PK \|RAIL7RMS \|RAIL7RMSFULL \|L2LPH1VRMS
          \|L2LPH1VCFactor \|L2LPH1TruePwr \|L2LPH1RePwr \|L2LPH1AppPwr \|L2LPH1PwrFactor
          \|L2LPH1PhaseDiff \|L2LPH1VPhase \|L2LPH2VRMS \|L2LPH2VCFactor \|L2LPH2TruePwr
          \|L2LPH2RePwr \|L2LPH2AppPwr \|L2LPH2PwrFactor \|L2LPH2PhaseDiff \|L2LPH2VPhase
          \|L2LPH3VRMS \|L2LPH3VCFactor \|L2LPH3TruePwr \|L2LPH3RePwr \|L2LPH3AppPwr
          \|L2LPH3PwrFactor \|L2LPH3PhaseDiff \|L2LPH3VPhase \|L2NPH1VRMS \|L2NPH1VCFactor
          \|L2NPH1TruePwr \|L2NPH1RePwr \|L2NPH1AppPwr \|L2NPH1PwrFactor \|L2NPH1PhaseDiff
          \|L2NPH1VPhase \|L2NPH2VRMS \|L2NPH2VCFactor \|L2NPH2TruePwr \|L2NPH2RePwr \|L2NPH2AppPwr
          \|L2NPH2PwrFactor \|L2NPH2PhaseDiff \|L2NPH2VPhase \|L2NPH3VRMS \|L2NPH3VCFactor
          \|L2NPH3TruePwr \|L2NPH3RePwr \|L2NPH3AppPwr \|L2NPH3PwrFactor \|L2NPH3PhaseDiff
          \|L2NPH3VPhase \|PH1IRMS \|PH1IPhase \|PH1ICFactor \|PH2IRMS \|PH2IPhase \|PH2ICFactor
          \|PH3IRMS \|PH3IPhase \|PH3ICFactor \|Frequency \|L2LPH1F1Mag \|L2LPH1F3Mag \|L2LPH1THDF
          \|L2LPH1THDR \|L2LPH1RMS \|L2LPH1IRMS \|L2LPH1Status \|L2LPH1HarmonicsNumber
          \|L2LPH1Frequency \|L2LPH1MagnitudeAbs \|L2LPH1MagnitudePct \|L2LPH1Phase \|L2LPH1Limits
          \|L2LPH1PassFail \|L2LPH1Margin \|L2LPH2F1Mag \|L2LPH2F3Mag \|L2LPH2THDF \|L2LPH2THDR
          \|L2LPH2RMS \|L2LPH2IRMS \|L2LPH2Status \|L2LPH2HarmonicsNumber \|L2LPH2Frequency
          \|L2LPH2MagnitudeAbs \|L2LPH2MagnitudePct \|L2LPH2Phase \|L2LPH2Limits \|L2LPH2PassFail
          \|L2LPH2Margin \|L2LPH3F1Mag \|L2LPH3F3Mag \|L2LPH3THDF \|L2LPH3THDR \|L2LPH3RMS
          \|L2LPH3IRMS \|L2LPH3Status \|L2LPH3HarmonicsNumber \|L2LPH3Frequency \|L2LPH3MagnitudeAbs
          \|L2LPH3MagnitudePct \|L2LPH3Phase \|L2LPH3Limits \|L2LPH3PassFail \|L2LPH3Margin
          \|L2NPH1F1Mag \|L2NPH1F3Mag \|L2NPH1THDF \|L2NPH1THDR \|L2NPH1RMS \|L2NPH1IRMS
          \|L2NPH1Status \|L2NPH1HarmonicsNumber \|L2NPH1Frequency \|L2NPH1MagnitudeAbs
          \|L2NPH1MagnitudePct \|L2NPH1Phase\|L2NPH1Limits \|L2NPH1PassFail\|L2NPH1Margin
          \|L2NPH2F1Mag\|L2NPH2F3Mag \|L2NPH2THDF\|L2NPH2THDR \|L2NPH2RMS\|L2NPH2IRMS \|L2NPH2Status
          \|L2NPH2HarmonicsNumber \|L2NPH2Frequency \|L2NPH2MagnitudeAbs \|L2NPH2MagnitudePct
          \|L2NPH2Phase\|L2NPH2Limits \|L2NPH2PassFail\|L2NPH2Margin \|L2NPH3F1Mag \|L2NPH3F3Mag
          \|L2NPH3THDF \|L2NPH3THDR \|L2NPH3RMS \|L2NPH3IRMS \|L2NPH3Status \|L2NPH3HarmonicsNumber
          \|L2NPH3Frequency \|L2NPH3MagnitudeAbs \|L2NPH3MagnitudePct \|L2NPH3Phase \|L2NPH3Limits
          \|L2NPH3PassFail \|L2NPH3Margin \|ORDER\|PH1INPwr \|PH1OUTPwr \|PH1Efficiency \|PH2INPwr
          \|PH2OUTPwr \|PH2Efficiency \|TotalEfficiency \|PH1LRIPRMS \|PH1LRIPPK2PK \|PH2LRIPRMS
          \|PH2LRIPPK2PK \|PH3LRIPRMS \|PH3LRIPPK2PK \|PH1SWRIPRMS \|PH1SWRIPPK2PK \|PH2SWRIPRMS
          \|PH2SWRIPPK2PK \|PH3SWRIPRMS \|PH3SWRIPPK2PK \|TruePwrSum \|RePwrSum \|AppPwrSum
          \|InPwrSum \|OutPwrSum.
    """


class MeasuMeas1SubgroupResultsCurrentacqMaximum(SCPICmdReadWithArguments):
    r"""The ``MEASU:MEAS1:SUBGROUP:RESUlts:CURRentacq:MAXimum`` command.

    Description:
        - This query returns the maximum value of the measurement specified by the string, for the
          current acquisition.

    Usage:
        - Using the ``.query(argument)`` method will send the
          ``MEASU:MEAS1:SUBGROUP:RESUlts:CURRentacq:MAXimum? argument`` query.
        - Using the ``.verify(argument, value)`` method will send the
          ``MEASU:MEAS1:SUBGROUP:RESUlts:CURRentacq:MAXimum? argument`` query and raise an
          AssertionError if the returned value does not match ``value``.

    SCPI Syntax:
        ```
        - MEASU:MEAS1:SUBGROUP:RESUlts:CURRentacq:MAXimum? <Qstring>
        ```

    Info:
        - ``<Qstring>`` = INPUT\|OUTPUT1\|OUTPUT2\|OUTPUT3\|OUTPUT4\|OUTPUT5\| OUTPUT6\|OUTPUT7\|
          RAIL1DPMOVERSHOOT\|RAIL1DPMUNDERSHOOT \|RAIL1FREQUENCY \|RAIL1PK2PK \|RAIL1RMS
          \|RAIL1RMSFULL\|RAIL2FREQUENCY \|RAIL2PK2PK \|RAIL2RMS \|RAIL2RMSFULL \|RAIL3FREQUENCY
          \|RAIL3PK2PK \|RAIL3RMS\|RAIL3RMSFULL \|RAIL4FREQUENCY \|RAIL4PK2PK \|RAIL4RMS
          \|RAIL4RMSFULL \|RAIL5FREQUENCY \|RAIL5PK2PK\|RAIL5RMS \|RAIL5RMSFULL \|RAIL6FREQUENCY
          \|RAIL6PK2PK \|RAIL6RMS \|RAIL6RMSFULL \|RAIL7DPMOVERSHOOT \|RAIL7DPMUNDERSHOOT
          \|RAIL7FREQUENCY \|RAIL7PK2PK \|RAIL7RMS \|RAIL7RMSFULL.
    """


class MeasuMeas1SubgroupResultsCurrentacq(SCPICmdRead):
    """The ``MEASU:MEAS1:SUBGROUP:RESUlts:CURRentacq`` command tree.

    Usage:
        - Using the ``.query()`` method will send the ``MEASU:MEAS1:SUBGROUP:RESUlts:CURRentacq?``
          query.
        - Using the ``.verify(value)`` method will send the
          ``MEASU:MEAS1:SUBGROUP:RESUlts:CURRentacq?`` query and raise an AssertionError if the
          returned value does not match ``value``.

    Properties:
        - ``.maximum``: The ``MEASU:MEAS1:SUBGROUP:RESUlts:CURRentacq:MAXimum`` command.
        - ``.mean``: The ``MEASU:MEAS1:SUBGROUP:RESUlts:CURRentacq:MEAN`` command.
        - ``.minimum``: The ``MEASU:MEAS1:SUBGROUP:RESUlts:CURRentacq:MINimum`` command.
        - ``.pk2pk``: The ``MEASU:MEAS1:SUBGROUP:RESUlts:CURRentacq:PK2PK`` command.
        - ``.population``: The ``MEASU:MEAS1:SUBGROUP:RESUlts:CURRentacq:POPUlation`` command.
        - ``.stddev``: The ``MEASU:MEAS1:SUBGROUP:RESUlts:CURRentacq:STDDev`` command.
    """

    def __init__(self, device: Optional["PIControl"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._maximum = MeasuMeas1SubgroupResultsCurrentacqMaximum(
            device, f"{self._cmd_syntax}:MAXimum"
        )
        self._mean = MeasuMeas1SubgroupResultsCurrentacqMean(device, f"{self._cmd_syntax}:MEAN")
        self._minimum = MeasuMeas1SubgroupResultsCurrentacqMinimum(
            device, f"{self._cmd_syntax}:MINimum"
        )
        self._pk2pk = MeasuMeas1SubgroupResultsCurrentacqPk2pk(device, f"{self._cmd_syntax}:PK2PK")
        self._population = MeasuMeas1SubgroupResultsCurrentacqPopulation(
            device, f"{self._cmd_syntax}:POPUlation"
        )
        self._stddev = MeasuMeas1SubgroupResultsCurrentacqStddev(
            device, f"{self._cmd_syntax}:STDDev"
        )

    @property
    def maximum(self) -> MeasuMeas1SubgroupResultsCurrentacqMaximum:
        r"""Return the ``MEASU:MEAS1:SUBGROUP:RESUlts:CURRentacq:MAXimum`` command.

        Description:
            - This query returns the maximum value of the measurement specified by the string, for
              the current acquisition.

        Usage:
            - Using the ``.query(argument)`` method will send the
              ``MEASU:MEAS1:SUBGROUP:RESUlts:CURRentacq:MAXimum? argument`` query.
            - Using the ``.verify(argument, value)`` method will send the
              ``MEASU:MEAS1:SUBGROUP:RESUlts:CURRentacq:MAXimum? argument`` query and raise an
              AssertionError if the returned value does not match ``value``.

        SCPI Syntax:
            ```
            - MEASU:MEAS1:SUBGROUP:RESUlts:CURRentacq:MAXimum? <Qstring>
            ```

        Info:
            - ``<Qstring>`` = INPUT\|OUTPUT1\|OUTPUT2\|OUTPUT3\|OUTPUT4\|OUTPUT5\|
              OUTPUT6\|OUTPUT7\| RAIL1DPMOVERSHOOT\|RAIL1DPMUNDERSHOOT \|RAIL1FREQUENCY \|RAIL1PK2PK
              \|RAIL1RMS \|RAIL1RMSFULL\|RAIL2FREQUENCY \|RAIL2PK2PK \|RAIL2RMS \|RAIL2RMSFULL
              \|RAIL3FREQUENCY \|RAIL3PK2PK \|RAIL3RMS\|RAIL3RMSFULL \|RAIL4FREQUENCY \|RAIL4PK2PK
              \|RAIL4RMS \|RAIL4RMSFULL \|RAIL5FREQUENCY \|RAIL5PK2PK\|RAIL5RMS \|RAIL5RMSFULL
              \|RAIL6FREQUENCY \|RAIL6PK2PK \|RAIL6RMS \|RAIL6RMSFULL \|RAIL7DPMOVERSHOOT
              \|RAIL7DPMUNDERSHOOT \|RAIL7FREQUENCY \|RAIL7PK2PK \|RAIL7RMS \|RAIL7RMSFULL.
        """
        return self._maximum

    @property
    def mean(self) -> MeasuMeas1SubgroupResultsCurrentacqMean:
        r"""Return the ``MEASU:MEAS1:SUBGROUP:RESUlts:CURRentacq:MEAN`` command.

        Description:
            - This query returns the mean value of the measurement specified by the string, for the
              current acquisition.

        Usage:
            - Using the ``.query(argument)`` method will send the
              ``MEASU:MEAS1:SUBGROUP:RESUlts:CURRentacq:MEAN? argument`` query.
            - Using the ``.verify(argument, value)`` method will send the
              ``MEASU:MEAS1:SUBGROUP:RESUlts:CURRentacq:MEAN? argument`` query and raise an
              AssertionError if the returned value does not match ``value``.

        SCPI Syntax:
            ```
            - MEASU:MEAS1:SUBGROUP:RESUlts:CURRentacq:MEAN? <Qstring>
            ```

        Info:
            - ``<Qstring>`` = INPUT \|OUTPUT1 \|OUTPUT2 \|OUTPUT3 \|OUTPUT4 \|OUTPUT5 \|OUTPUT6
              \|OUTPUT7 \|RAIL1DPMOVERSHOOT \|RAIL1DPMUNDERSHOOT \|RAIL1FREQUENCY \|RAIL1PK2PK
              \|RAIL1RMS \|RAIL1RMSFULL \|RAIL2FREQUENCY \|RAIL2PK2PK \|RAIL2RMS \|RAIL2RMSFULL
              \|RAIL3FREQUENCY \|RAIL3PK2PK \|RAIL3RMS \|RAIL3RMSFULL \|RAIL4FREQUENCY \|RAIL4PK2PK
              \|RAIL4RMS \|RAIL4RMSFULL \|RAIL5FREQUENCY \|RAIL5PK2PK \|RAIL5RMS \|RAIL5RMSFULL
              \|RAIL6FREQUENCY \|RAIL6PK2PK \|RAIL6RMS \|RAIL6RMSFULL \|RAIL7DPMOVERSHOOT
              \|RAIL7DPMUNDERSHOOT \|RAIL7FREQUENCY \|RAIL7PK2PK \|RAIL7RMS \|RAIL7RMSFULL
              \|L2LPH1VRMS \|L2LPH1VCFactor \|L2LPH1TruePwr \|L2LPH1RePwr \|L2LPH1AppPwr
              \|L2LPH1PwrFactor \|L2LPH1PhaseDiff \|L2LPH1VPhase \|L2LPH2VRMS \|L2LPH2VCFactor
              \|L2LPH2TruePwr \|L2LPH2RePwr \|L2LPH2AppPwr \|L2LPH2PwrFactor \|L2LPH2PhaseDiff
              \|L2LPH2VPhase \|L2LPH3VRMS \|L2LPH3VCFactor \|L2LPH3TruePwr \|L2LPH3RePwr
              \|L2LPH3AppPwr \|L2LPH3PwrFactor \|L2LPH3PhaseDiff \|L2LPH3VPhase \|L2NPH1VRMS
              \|L2NPH1VCFactor \|L2NPH1TruePwr \|L2NPH1RePwr \|L2NPH1AppPwr \|L2NPH1PwrFactor
              \|L2NPH1PhaseDiff \|L2NPH1VPhase \|L2NPH2VRMS \|L2NPH2VCFactor \|L2NPH2TruePwr
              \|L2NPH2RePwr \|L2NPH2AppPwr \|L2NPH2PwrFactor \|L2NPH2PhaseDiff \|L2NPH2VPhase
              \|L2NPH3VRMS \|L2NPH3VCFactor \|L2NPH3TruePwr \|L2NPH3RePwr \|L2NPH3AppPwr
              \|L2NPH3PwrFactor \|L2NPH3PhaseDiff \|L2NPH3VPhase \|PH1IRMS \|PH1IPhase \|PH1ICFactor
              \|PH2IRMS \|PH2IPhase \|PH2ICFactor \|PH3IRMS \|PH3IPhase \|PH3ICFactor \|Frequency
              \|L2LPH1F1Mag \|L2LPH1F3Mag \|L2LPH1THDF \|L2LPH1THDR \|L2LPH1RMS \|L2LPH1IRMS
              \|L2LPH1Status \|L2LPH1HarmonicsNumber \|L2LPH1Frequency \|L2LPH1MagnitudeAbs
              \|L2LPH1MagnitudePct \|L2LPH1Phase \|L2LPH1Limits \|L2LPH1PassFail \|L2LPH1Margin
              \|L2LPH2F1Mag \|L2LPH2F3Mag \|L2LPH2THDF \|L2LPH2THDR \|L2LPH2RMS \|L2LPH2IRMS
              \|L2LPH2Status \|L2LPH2HarmonicsNumber \|L2LPH2Frequency \|L2LPH2MagnitudeAbs
              \|L2LPH2MagnitudePct \|L2LPH2Phase \|L2LPH2Limits \|L2LPH2PassFail \|L2LPH2Margin
              \|L2LPH3F1Mag \|L2LPH3F3Mag \|L2LPH3THDF \|L2LPH3THDR \|L2LPH3RMS \|L2LPH3IRMS
              \|L2LPH3Status \|L2LPH3HarmonicsNumber \|L2LPH3Frequency \|L2LPH3MagnitudeAbs
              \|L2LPH3MagnitudePct \|L2LPH3Phase \|L2LPH3Limits \|L2LPH3PassFail \|L2LPH3Margin
              \|L2NPH1F1Mag \|L2NPH1F3Mag \|L2NPH1THDF \|L2NPH1THDR \|L2NPH1RMS \|L2NPH1IRMS
              \|L2NPH1Status \|L2NPH1HarmonicsNumber \|L2NPH1Frequency \|L2NPH1MagnitudeAbs
              \|L2NPH1MagnitudePct \|L2NPH1Phase\|L2NPH1Limits \|L2NPH1PassFail\|L2NPH1Margin
              \|L2NPH2F1Mag\|L2NPH2F3Mag \|L2NPH2THDF\|L2NPH2THDR \|L2NPH2RMS\|L2NPH2IRMS
              \|L2NPH2Status \|L2NPH2HarmonicsNumber \|L2NPH2Frequency \|L2NPH2MagnitudeAbs
              \|L2NPH2MagnitudePct \|L2NPH2Phase\|L2NPH2Limits \|L2NPH2PassFail\|L2NPH2Margin
              \|L2NPH3F1Mag \|L2NPH3F3Mag \|L2NPH3THDF \|L2NPH3THDR \|L2NPH3RMS \|L2NPH3IRMS
              \|L2NPH3Status \|L2NPH3HarmonicsNumber \|L2NPH3Frequency \|L2NPH3MagnitudeAbs
              \|L2NPH3MagnitudePct \|L2NPH3Phase \|L2NPH3Limits \|L2NPH3PassFail \|L2NPH3Margin
              \|ORDER\|PH1INPwr \|PH1OUTPwr \|PH1Efficiency \|PH2INPwr \|PH2OUTPwr \|PH2Efficiency
              \|TotalEfficiency \|PH1LRIPRMS \|PH1LRIPPK2PK \|PH2LRIPRMS \|PH2LRIPPK2PK \|PH3LRIPRMS
              \|PH3LRIPPK2PK \|PH1SWRIPRMS \|PH1SWRIPPK2PK \|PH2SWRIPRMS \|PH2SWRIPPK2PK
              \|PH3SWRIPRMS \|PH3SWRIPPK2PK \|TruePwrSum \|RePwrSum \|AppPwrSum \|InPwrSum
              \|OutPwrSum.
        """
        return self._mean

    @property
    def minimum(self) -> MeasuMeas1SubgroupResultsCurrentacqMinimum:
        r"""Return the ``MEASU:MEAS1:SUBGROUP:RESUlts:CURRentacq:MINimum`` command.

        Description:
            - This query returns the minimum value of the measurement specified by the string, for
              the current acquisition.

        Usage:
            - Using the ``.query(argument)`` method will send the
              ``MEASU:MEAS1:SUBGROUP:RESUlts:CURRentacq:MINimum? argument`` query.
            - Using the ``.verify(argument, value)`` method will send the
              ``MEASU:MEAS1:SUBGROUP:RESUlts:CURRentacq:MINimum? argument`` query and raise an
              AssertionError if the returned value does not match ``value``.

        SCPI Syntax:
            ```
            - MEASU:MEAS1:SUBGROUP:RESUlts:CURRentacq:MINimum? <Qstring>
            ```

        Info:
            - ``<Qstring>`` = INPUT\|OUTPUT1\|OUTPUT2\|OUTPUT3\|OUTPUT4\|OUTPUT5\|
              OUTPUT6\|OUTPUT7\| RAIL1DPMOVERSHOOT\|RAIL1DPMUNDERSHOOT \|RAIL1FREQUENCY \|RAIL1PK2PK
              \|RAIL1RMS \|RAIL1RMSFULL\|RAIL2FREQUENCY \|RAIL2PK2PK \|RAIL2RMS \|RAIL2RMSFULL
              \|RAIL3FREQUENCY \|RAIL3PK2PK \|RAIL3RMS\|RAIL3RMSFULL \|RAIL4FREQUENCY \|RAIL4PK2PK
              \|RAIL4RMS \|RAIL4RMSFULL \|RAIL5FREQUENCY \|RAIL5PK2PK\|RAIL5RMS \|RAIL5RMSFULL
              \|RAIL6FREQUENCY \|RAIL6PK2PK \|RAIL6RMS \|RAIL6RMSFULL \|RAIL7DPMOVERSHOOT
              \|RAIL7DPMUNDERSHOOT \|RAIL7FREQUENCY \|RAIL7PK2PK \|RAIL7RMS \|RAIL7RMSFULL.
        """
        return self._minimum

    @property
    def pk2pk(self) -> MeasuMeas1SubgroupResultsCurrentacqPk2pk:
        r"""Return the ``MEASU:MEAS1:SUBGROUP:RESUlts:CURRentacq:PK2PK`` command.

        Description:
            - This query returns the peak-to-peak value of the measurement specified by the string,
              for the current acquisition.

        Usage:
            - Using the ``.query(argument)`` method will send the
              ``MEASU:MEAS1:SUBGROUP:RESUlts:CURRentacq:PK2PK? argument`` query.
            - Using the ``.verify(argument, value)`` method will send the
              ``MEASU:MEAS1:SUBGROUP:RESUlts:CURRentacq:PK2PK? argument`` query and raise an
              AssertionError if the returned value does not match ``value``.

        SCPI Syntax:
            ```
            - MEASU:MEAS1:SUBGROUP:RESUlts:CURRentacq:PK2PK? <Qstring>
            ```

        Info:
            - ``<Qstring>`` = INPUT\|OUTPUT1\|OUTPUT2\|OUTPUT3\|OUTPUT4\|OUTPUT5\|
              OUTPUT6\|OUTPUT7\| RAIL1DPMOVERSHOOT\|RAIL1DPMUNDERSHOOT \|RAIL1FREQUENCY \|RAIL1PK2PK
              \|RAIL1RMS \|RAIL1RMSFULL\|RAIL2FREQUENCY \|RAIL2PK2PK \|RAIL2RMS \|RAIL2RMSFULL
              \|RAIL3FREQUENCY \|RAIL3PK2PK \|RAIL3RMS\|RAIL3RMSFULL \|RAIL4FREQUENCY \|RAIL4PK2PK
              \|RAIL4RMS \|RAIL4RMSFULL \|RAIL5FREQUENCY \|RAIL5PK2PK\|RAIL5RMS \|RAIL5RMSFULL
              \|RAIL6FREQUENCY \|RAIL6PK2PK \|RAIL6RMS \|RAIL6RMSFULL \|RAIL7DPMOVERSHOOT
              \|RAIL7DPMUNDERSHOOT \|RAIL7FREQUENCY \|RAIL7PK2PK \|RAIL7RMS \|RAIL7RMSFULL.
        """
        return self._pk2pk

    @property
    def population(self) -> MeasuMeas1SubgroupResultsCurrentacqPopulation:
        r"""Return the ``MEASU:MEAS1:SUBGROUP:RESUlts:CURRentacq:POPUlation`` command.

        Description:
            - This query returns the population value of the measurement specified by the string,
              for the current acquisition.

        Usage:
            - Using the ``.query(argument)`` method will send the
              ``MEASU:MEAS1:SUBGROUP:RESUlts:CURRentacq:POPUlation? argument`` query.
            - Using the ``.verify(argument, value)`` method will send the
              ``MEASU:MEAS1:SUBGROUP:RESUlts:CURRentacq:POPUlation? argument`` query and raise an
              AssertionError if the returned value does not match ``value``.

        SCPI Syntax:
            ```
            - MEASU:MEAS1:SUBGROUP:RESUlts:CURRentacq:POPUlation? <Qstring>
            ```

        Info:
            - ``<Qstring>`` = INPUT\|OUTPUT1\|OUTPUT2\|OUTPUT3\|OUTPUT4\|OUTPUT5\|
              OUTPUT6\|OUTPUT7\| RAIL1DPMOVERSHOOT\|RAIL1DPMUNDERSHOOT \|RAIL1FREQUENCY \|RAIL1PK2PK
              \|RAIL1RMS \|RAIL1RMSFULL\|RAIL2FREQUENCY \|RAIL2PK2PK \|RAIL2RMS \|RAIL2RMSFULL
              \|RAIL3FREQUENCY \|RAIL3PK2PK \|RAIL3RMS\|RAIL3RMSFULL \|RAIL4FREQUENCY \|RAIL4PK2PK
              \|RAIL4RMS \|RAIL4RMSFULL \|RAIL5FREQUENCY \|RAIL5PK2PK\|RAIL5RMS \|RAIL5RMSFULL
              \|RAIL6FREQUENCY \|RAIL6PK2PK \|RAIL6RMS \|RAIL6RMSFULL \|RAIL7DPMOVERSHOOT
              \|RAIL7DPMUNDERSHOOT \|RAIL7FREQUENCY \|RAIL7PK2PK \|RAIL7RMS \|RAIL7RMSFULL.
        """
        return self._population

    @property
    def stddev(self) -> MeasuMeas1SubgroupResultsCurrentacqStddev:
        r"""Return the ``MEASU:MEAS1:SUBGROUP:RESUlts:CURRentacq:STDDev`` command.

        Description:
            - This query returns the standard deviation value of the measurement specified by the
              string, for the current acquisition.

        Usage:
            - Using the ``.query(argument)`` method will send the
              ``MEASU:MEAS1:SUBGROUP:RESUlts:CURRentacq:STDDev? argument`` query.
            - Using the ``.verify(argument, value)`` method will send the
              ``MEASU:MEAS1:SUBGROUP:RESUlts:CURRentacq:STDDev? argument`` query and raise an
              AssertionError if the returned value does not match ``value``.

        SCPI Syntax:
            ```
            - MEASU:MEAS1:SUBGROUP:RESUlts:CURRentacq:STDDev? <Qstring>
            ```

        Info:
            - ``<Qstring>`` = INPUT\|OUTPUT1\|OUTPUT2\|OUTPUT3\|OUTPUT4\|OUTPUT5\|
              OUTPUT6\|OUTPUT7\| RAIL1DPMOVERSHOOT\|RAIL1DPMUNDERSHOOT \|RAIL1FREQUENCY \|RAIL1PK2PK
              \|RAIL1RMS \|RAIL1RMSFULL\|RAIL2FREQUENCY \|RAIL2PK2PK \|RAIL2RMS \|RAIL2RMSFULL
              \|RAIL3FREQUENCY \|RAIL3PK2PK \|RAIL3RMS\|RAIL3RMSFULL \|RAIL4FREQUENCY \|RAIL4PK2PK
              \|RAIL4RMS \|RAIL4RMSFULL \|RAIL5FREQUENCY \|RAIL5PK2PK\|RAIL5RMS \|RAIL5RMSFULL
              \|RAIL6FREQUENCY \|RAIL6PK2PK \|RAIL6RMS \|RAIL6RMSFULL \|RAIL7DPMOVERSHOOT
              \|RAIL7DPMUNDERSHOOT \|RAIL7FREQUENCY \|RAIL7PK2PK \|RAIL7RMS \|RAIL7RMSFULL.
        """
        return self._stddev


class MeasuMeas1SubgroupResultsAllacqsStddev(SCPICmdReadWithArguments):
    r"""The ``MEASU:MEAS1:SUBGROUP:RESUlts:ALLAcqs:STDDev`` command.

    Description:
        - This query returns the standard deviation value of the measurement specified by the
          string, for all acquisitions.

    Usage:
        - Using the ``.query(argument)`` method will send the
          ``MEASU:MEAS1:SUBGROUP:RESUlts:ALLAcqs:STDDev? argument`` query.
        - Using the ``.verify(argument, value)`` method will send the
          ``MEASU:MEAS1:SUBGROUP:RESUlts:ALLAcqs:STDDev? argument`` query and raise an
          AssertionError if the returned value does not match ``value``.

    SCPI Syntax:
        ```
        - MEASU:MEAS1:SUBGROUP:RESUlts:ALLAcqs:STDDev? <Qstring>
        ```

    Info:
        - ``<Qstring>`` = INPUT\|OUTPUT1\|OUTPUT2\|OUTPUT3\|OUTPUT4\|OUTPUT5\| OUTPUT6\|OUTPUT7\|
          RAIL1DPMOVERSHOOT\|RAIL1DPMUNDERSHOOT \|RAIL1FREQUENCY \|RAIL1PK2PK \|RAIL1RMS
          \|RAIL1RMSFULL\|RAIL2FREQUENCY \|RAIL2PK2PK \|RAIL2RMS \|RAIL2RMSFULL \|RAIL3FREQUENCY
          \|RAIL3PK2PK \|RAIL3RMS\|RAIL3RMSFULL \|RAIL4FREQUENCY \|RAIL4PK2PK \|RAIL4RMS
          \|RAIL4RMSFULL \|RAIL5FREQUENCY \|RAIL5PK2PK\|RAIL5RMS \|RAIL5RMSFULL \|RAIL6FREQUENCY
          \|RAIL6PK2PK \|RAIL6RMS \|RAIL6RMSFULL \|RAIL7DPMOVERSHOOT \|RAIL7DPMUNDERSHOOT
          \|RAIL7FREQUENCY \|RAIL7PK2PK \|RAIL7RMS \|RAIL7RMSFULL.
    """


class MeasuMeas1SubgroupResultsAllacqsPopulation(SCPICmdReadWithArguments):
    r"""The ``MEASU:MEAS1:SUBGROUP:RESUlts:ALLAcqs:POPUlation`` command.

    Description:
        - This query returns the population value of the measurement specified by the string, for
          all acquisitions.

    Usage:
        - Using the ``.query(argument)`` method will send the
          ``MEASU:MEAS1:SUBGROUP:RESUlts:ALLAcqs:POPUlation? argument`` query.
        - Using the ``.verify(argument, value)`` method will send the
          ``MEASU:MEAS1:SUBGROUP:RESUlts:ALLAcqs:POPUlation? argument`` query and raise an
          AssertionError if the returned value does not match ``value``.

    SCPI Syntax:
        ```
        - MEASU:MEAS1:SUBGROUP:RESUlts:ALLAcqs:POPUlation? <Qstring>
        ```

    Info:
        - ``<Qstring>`` = INPUT\|OUTPUT1\|OUTPUT2\|OUTPUT3\|OUTPUT4\|OUTPUT5\| OUTPUT6\|OUTPUT7\|
          RAIL1DPMOVERSHOOT\|RAIL1DPMUNDERSHOOT \|RAIL1FREQUENCY \|RAIL1PK2PK \|RAIL1RMS
          \|RAIL1RMSFULL\|RAIL2FREQUENCY \|RAIL2PK2PK \|RAIL2RMS \|RAIL2RMSFULL \|RAIL3FREQUENCY
          \|RAIL3PK2PK \|RAIL3RMS\|RAIL3RMSFULL \|RAIL4FREQUENCY \|RAIL4PK2PK \|RAIL4RMS
          \|RAIL4RMSFULL \|RAIL5FREQUENCY \|RAIL5PK2PK\|RAIL5RMS \|RAIL5RMSFULL \|RAIL6FREQUENCY
          \|RAIL6PK2PK \|RAIL6RMS \|RAIL6RMSFULL \|RAIL7DPMOVERSHOOT \|RAIL7DPMUNDERSHOOT
          \|RAIL7FREQUENCY \|RAIL7PK2PK \|RAIL7RMS \|RAIL7RMSFULL.
    """


class MeasuMeas1SubgroupResultsAllacqsPk2pk(SCPICmdReadWithArguments):
    r"""The ``MEASU:MEAS1:SUBGROUP:RESUlts:ALLAcqs:PK2PK`` command.

    Description:
        - This query returns the peak-to-peak value of the measurement specified by the string, for
          all acquisitions.

    Usage:
        - Using the ``.query(argument)`` method will send the
          ``MEASU:MEAS1:SUBGROUP:RESUlts:ALLAcqs:PK2PK? argument`` query.
        - Using the ``.verify(argument, value)`` method will send the
          ``MEASU:MEAS1:SUBGROUP:RESUlts:ALLAcqs:PK2PK? argument`` query and raise an AssertionError
          if the returned value does not match ``value``.

    SCPI Syntax:
        ```
        - MEASU:MEAS1:SUBGROUP:RESUlts:ALLAcqs:PK2PK? <Qstring>
        ```

    Info:
        - ``<Qstring>`` = INPUT\|OUTPUT1\|OUTPUT2\|OUTPUT3\|OUTPUT4\|OUTPUT5\| OUTPUT6\|OUTPUT7\|
          RAIL1DPMOVERSHOOT\|RAIL1DPMUNDERSHOOT \|RAIL1FREQUENCY \|RAIL1PK2PK \|RAIL1RMS
          \|RAIL1RMSFULL\|RAIL2FREQUENCY \|RAIL2PK2PK \|RAIL2RMS \|RAIL2RMSFULL \|RAIL3FREQUENCY
          \|RAIL3PK2PK \|RAIL3RMS\|RAIL3RMSFULL \|RAIL4FREQUENCY \|RAIL4PK2PK \|RAIL4RMS
          \|RAIL4RMSFULL \|RAIL5FREQUENCY \|RAIL5PK2PK\|RAIL5RMS \|RAIL5RMSFULL \|RAIL6FREQUENCY
          \|RAIL6PK2PK \|RAIL6RMS \|RAIL6RMSFULL \|RAIL7DPMOVERSHOOT \|RAIL7DPMUNDERSHOOT
          \|RAIL7FREQUENCY \|RAIL7PK2PK \|RAIL7RMS \|RAIL7RMSFULL.
    """


class MeasuMeas1SubgroupResultsAllacqsMinimum(SCPICmdReadWithArguments):
    r"""The ``MEASU:MEAS1:SUBGROUP:RESUlts:ALLAcqs:MINimum`` command.

    Description:
        - This query returns the minimum value of the measurement specified by the string, for all
          acquisitions.

    Usage:
        - Using the ``.query(argument)`` method will send the
          ``MEASU:MEAS1:SUBGROUP:RESUlts:ALLAcqs:MINimum? argument`` query.
        - Using the ``.verify(argument, value)`` method will send the
          ``MEASU:MEAS1:SUBGROUP:RESUlts:ALLAcqs:MINimum? argument`` query and raise an
          AssertionError if the returned value does not match ``value``.

    SCPI Syntax:
        ```
        - MEASU:MEAS1:SUBGROUP:RESUlts:ALLAcqs:MINimum? <Qstring>
        ```

    Info:
        - ``<Qstring>`` = INPUT\|OUTPUT1\|OUTPUT2\|OUTPUT3\|OUTPUT4\|OUTPUT5\| OUTPUT6\|OUTPUT7\|
          RAIL1DPMOVERSHOOT\|RAIL1DPMUNDERSHOOT \|RAIL1FREQUENCY \|RAIL1PK2PK \|RAIL1RMS
          \|RAIL1RMSFULL\|RAIL2FREQUENCY \|RAIL2PK2PK \|RAIL2RMS \|RAIL2RMSFULL \|RAIL3FREQUENCY
          \|RAIL3PK2PK \|RAIL3RMS\|RAIL3RMSFULL \|RAIL4FREQUENCY \|RAIL4PK2PK \|RAIL4RMS
          \|RAIL4RMSFULL \|RAIL5FREQUENCY \|RAIL5PK2PK\|RAIL5RMS \|RAIL5RMSFULL \|RAIL6FREQUENCY
          \|RAIL6PK2PK \|RAIL6RMS \|RAIL6RMSFULL \|RAIL7DPMOVERSHOOT \|RAIL7DPMUNDERSHOOT
          \|RAIL7FREQUENCY \|RAIL7PK2PK \|RAIL7RMS \|RAIL7RMSFULL.
    """


class MeasuMeas1SubgroupResultsAllacqsMean(SCPICmdReadWithArguments):
    r"""The ``MEASU:MEAS1:SUBGROUP:RESUlts:ALLAcqs:MEAN`` command.

    Description:
        - This query returns the mean value of the measurement specified by the string, for all
          acquisitions.

    Usage:
        - Using the ``.query(argument)`` method will send the
          ``MEASU:MEAS1:SUBGROUP:RESUlts:ALLAcqs:MEAN? argument`` query.
        - Using the ``.verify(argument, value)`` method will send the
          ``MEASU:MEAS1:SUBGROUP:RESUlts:ALLAcqs:MEAN? argument`` query and raise an AssertionError
          if the returned value does not match ``value``.

    SCPI Syntax:
        ```
        - MEASU:MEAS1:SUBGROUP:RESUlts:ALLAcqs:MEAN? <Qstring>
        ```

    Info:
        - ``<Qstring>`` = INPUT\|OUTPUT1\|OUTPUT2\|OUTPUT3\|OUTPUT4\|OUTPUT5\| OUTPUT6\|OUTPUT7\|
          RAIL1DPMOVERSHOOT\|RAIL1DPMUNDERSHOOT \|RAIL1FREQUENCY \|RAIL1PK2PK \|RAIL1RMS
          \|RAIL1RMSFULL\|RAIL2FREQUENCY \|RAIL2PK2PK \|RAIL2RMS \|RAIL2RMSFULL \|RAIL3FREQUENCY
          \|RAIL3PK2PK \|RAIL3RMS\|RAIL3RMSFULL \|RAIL4FREQUENCY \|RAIL4PK2PK \|RAIL4RMS
          \|RAIL4RMSFULL \|RAIL5FREQUENCY \|RAIL5PK2PK\|RAIL5RMS \|RAIL5RMSFULL \|RAIL6FREQUENCY
          \|RAIL6PK2PK \|RAIL6RMS \|RAIL6RMSFULL \|RAIL7DPMOVERSHOOT \|RAIL7DPMUNDERSHOOT
          \|RAIL7FREQUENCY \|RAIL7PK2PK \|RAIL7RMS \|RAIL7RMSFULL.
    """


class MeasuMeas1SubgroupResultsAllacqsMaximum(SCPICmdReadWithArguments):
    r"""The ``MEASU:MEAS1:SUBGROUP:RESUlts:ALLAcqs:MAXimum`` command.

    Description:
        - This query returns the maximum value of the measurement specified by the string, for all
          acquisitions.

    Usage:
        - Using the ``.query(argument)`` method will send the
          ``MEASU:MEAS1:SUBGROUP:RESUlts:ALLAcqs:MAXimum? argument`` query.
        - Using the ``.verify(argument, value)`` method will send the
          ``MEASU:MEAS1:SUBGROUP:RESUlts:ALLAcqs:MAXimum? argument`` query and raise an
          AssertionError if the returned value does not match ``value``.

    SCPI Syntax:
        ```
        - MEASU:MEAS1:SUBGROUP:RESUlts:ALLAcqs:MAXimum? <Qstring>
        ```

    Info:
        - ``<Qstring>`` = INPUT\|OUTPUT1\|OUTPUT2\|OUTPUT3\|OUTPUT4\|OUTPUT5\| OUTPUT6\|OUTPUT7\|
          RAIL1DPMOVERSHOOT\|RAIL1DPMUNDERSHOOT \|RAIL1FREQUENCY \|RAIL1PK2PK \|RAIL1RMS
          \|RAIL1RMSFULL\|RAIL2FREQUENCY \|RAIL2PK2PK \|RAIL2RMS \|RAIL2RMSFULL \|RAIL3FREQUENCY
          \|RAIL3PK2PK \|RAIL3RMS\|RAIL3RMSFULL \|RAIL4FREQUENCY \|RAIL4PK2PK \|RAIL4RMS
          \|RAIL4RMSFULL \|RAIL5FREQUENCY \|RAIL5PK2PK\|RAIL5RMS \|RAIL5RMSFULL \|RAIL6FREQUENCY
          \|RAIL6PK2PK \|RAIL6RMS \|RAIL6RMSFULL \|RAIL7DPMOVERSHOOT \|RAIL7DPMUNDERSHOOT
          \|RAIL7FREQUENCY \|RAIL7PK2PK \|RAIL7RMS \|RAIL7RMSFULL.
    """


class MeasuMeas1SubgroupResultsAllacqs(SCPICmdRead):
    """The ``MEASU:MEAS1:SUBGROUP:RESUlts:ALLAcqs`` command tree.

    Usage:
        - Using the ``.query()`` method will send the ``MEASU:MEAS1:SUBGROUP:RESUlts:ALLAcqs?``
          query.
        - Using the ``.verify(value)`` method will send the
          ``MEASU:MEAS1:SUBGROUP:RESUlts:ALLAcqs?`` query and raise an AssertionError if the
          returned value does not match ``value``.

    Properties:
        - ``.maximum``: The ``MEASU:MEAS1:SUBGROUP:RESUlts:ALLAcqs:MAXimum`` command.
        - ``.mean``: The ``MEASU:MEAS1:SUBGROUP:RESUlts:ALLAcqs:MEAN`` command.
        - ``.minimum``: The ``MEASU:MEAS1:SUBGROUP:RESUlts:ALLAcqs:MINimum`` command.
        - ``.pk2pk``: The ``MEASU:MEAS1:SUBGROUP:RESUlts:ALLAcqs:PK2PK`` command.
        - ``.population``: The ``MEASU:MEAS1:SUBGROUP:RESUlts:ALLAcqs:POPUlation`` command.
        - ``.stddev``: The ``MEASU:MEAS1:SUBGROUP:RESUlts:ALLAcqs:STDDev`` command.
    """

    def __init__(self, device: Optional["PIControl"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._maximum = MeasuMeas1SubgroupResultsAllacqsMaximum(
            device, f"{self._cmd_syntax}:MAXimum"
        )
        self._mean = MeasuMeas1SubgroupResultsAllacqsMean(device, f"{self._cmd_syntax}:MEAN")
        self._minimum = MeasuMeas1SubgroupResultsAllacqsMinimum(
            device, f"{self._cmd_syntax}:MINimum"
        )
        self._pk2pk = MeasuMeas1SubgroupResultsAllacqsPk2pk(device, f"{self._cmd_syntax}:PK2PK")
        self._population = MeasuMeas1SubgroupResultsAllacqsPopulation(
            device, f"{self._cmd_syntax}:POPUlation"
        )
        self._stddev = MeasuMeas1SubgroupResultsAllacqsStddev(device, f"{self._cmd_syntax}:STDDev")

    @property
    def maximum(self) -> MeasuMeas1SubgroupResultsAllacqsMaximum:
        r"""Return the ``MEASU:MEAS1:SUBGROUP:RESUlts:ALLAcqs:MAXimum`` command.

        Description:
            - This query returns the maximum value of the measurement specified by the string, for
              all acquisitions.

        Usage:
            - Using the ``.query(argument)`` method will send the
              ``MEASU:MEAS1:SUBGROUP:RESUlts:ALLAcqs:MAXimum? argument`` query.
            - Using the ``.verify(argument, value)`` method will send the
              ``MEASU:MEAS1:SUBGROUP:RESUlts:ALLAcqs:MAXimum? argument`` query and raise an
              AssertionError if the returned value does not match ``value``.

        SCPI Syntax:
            ```
            - MEASU:MEAS1:SUBGROUP:RESUlts:ALLAcqs:MAXimum? <Qstring>
            ```

        Info:
            - ``<Qstring>`` = INPUT\|OUTPUT1\|OUTPUT2\|OUTPUT3\|OUTPUT4\|OUTPUT5\|
              OUTPUT6\|OUTPUT7\| RAIL1DPMOVERSHOOT\|RAIL1DPMUNDERSHOOT \|RAIL1FREQUENCY \|RAIL1PK2PK
              \|RAIL1RMS \|RAIL1RMSFULL\|RAIL2FREQUENCY \|RAIL2PK2PK \|RAIL2RMS \|RAIL2RMSFULL
              \|RAIL3FREQUENCY \|RAIL3PK2PK \|RAIL3RMS\|RAIL3RMSFULL \|RAIL4FREQUENCY \|RAIL4PK2PK
              \|RAIL4RMS \|RAIL4RMSFULL \|RAIL5FREQUENCY \|RAIL5PK2PK\|RAIL5RMS \|RAIL5RMSFULL
              \|RAIL6FREQUENCY \|RAIL6PK2PK \|RAIL6RMS \|RAIL6RMSFULL \|RAIL7DPMOVERSHOOT
              \|RAIL7DPMUNDERSHOOT \|RAIL7FREQUENCY \|RAIL7PK2PK \|RAIL7RMS \|RAIL7RMSFULL.
        """
        return self._maximum

    @property
    def mean(self) -> MeasuMeas1SubgroupResultsAllacqsMean:
        r"""Return the ``MEASU:MEAS1:SUBGROUP:RESUlts:ALLAcqs:MEAN`` command.

        Description:
            - This query returns the mean value of the measurement specified by the string, for all
              acquisitions.

        Usage:
            - Using the ``.query(argument)`` method will send the
              ``MEASU:MEAS1:SUBGROUP:RESUlts:ALLAcqs:MEAN? argument`` query.
            - Using the ``.verify(argument, value)`` method will send the
              ``MEASU:MEAS1:SUBGROUP:RESUlts:ALLAcqs:MEAN? argument`` query and raise an
              AssertionError if the returned value does not match ``value``.

        SCPI Syntax:
            ```
            - MEASU:MEAS1:SUBGROUP:RESUlts:ALLAcqs:MEAN? <Qstring>
            ```

        Info:
            - ``<Qstring>`` = INPUT\|OUTPUT1\|OUTPUT2\|OUTPUT3\|OUTPUT4\|OUTPUT5\|
              OUTPUT6\|OUTPUT7\| RAIL1DPMOVERSHOOT\|RAIL1DPMUNDERSHOOT \|RAIL1FREQUENCY \|RAIL1PK2PK
              \|RAIL1RMS \|RAIL1RMSFULL\|RAIL2FREQUENCY \|RAIL2PK2PK \|RAIL2RMS \|RAIL2RMSFULL
              \|RAIL3FREQUENCY \|RAIL3PK2PK \|RAIL3RMS\|RAIL3RMSFULL \|RAIL4FREQUENCY \|RAIL4PK2PK
              \|RAIL4RMS \|RAIL4RMSFULL \|RAIL5FREQUENCY \|RAIL5PK2PK\|RAIL5RMS \|RAIL5RMSFULL
              \|RAIL6FREQUENCY \|RAIL6PK2PK \|RAIL6RMS \|RAIL6RMSFULL \|RAIL7DPMOVERSHOOT
              \|RAIL7DPMUNDERSHOOT \|RAIL7FREQUENCY \|RAIL7PK2PK \|RAIL7RMS \|RAIL7RMSFULL.
        """
        return self._mean

    @property
    def minimum(self) -> MeasuMeas1SubgroupResultsAllacqsMinimum:
        r"""Return the ``MEASU:MEAS1:SUBGROUP:RESUlts:ALLAcqs:MINimum`` command.

        Description:
            - This query returns the minimum value of the measurement specified by the string, for
              all acquisitions.

        Usage:
            - Using the ``.query(argument)`` method will send the
              ``MEASU:MEAS1:SUBGROUP:RESUlts:ALLAcqs:MINimum? argument`` query.
            - Using the ``.verify(argument, value)`` method will send the
              ``MEASU:MEAS1:SUBGROUP:RESUlts:ALLAcqs:MINimum? argument`` query and raise an
              AssertionError if the returned value does not match ``value``.

        SCPI Syntax:
            ```
            - MEASU:MEAS1:SUBGROUP:RESUlts:ALLAcqs:MINimum? <Qstring>
            ```

        Info:
            - ``<Qstring>`` = INPUT\|OUTPUT1\|OUTPUT2\|OUTPUT3\|OUTPUT4\|OUTPUT5\|
              OUTPUT6\|OUTPUT7\| RAIL1DPMOVERSHOOT\|RAIL1DPMUNDERSHOOT \|RAIL1FREQUENCY \|RAIL1PK2PK
              \|RAIL1RMS \|RAIL1RMSFULL\|RAIL2FREQUENCY \|RAIL2PK2PK \|RAIL2RMS \|RAIL2RMSFULL
              \|RAIL3FREQUENCY \|RAIL3PK2PK \|RAIL3RMS\|RAIL3RMSFULL \|RAIL4FREQUENCY \|RAIL4PK2PK
              \|RAIL4RMS \|RAIL4RMSFULL \|RAIL5FREQUENCY \|RAIL5PK2PK\|RAIL5RMS \|RAIL5RMSFULL
              \|RAIL6FREQUENCY \|RAIL6PK2PK \|RAIL6RMS \|RAIL6RMSFULL \|RAIL7DPMOVERSHOOT
              \|RAIL7DPMUNDERSHOOT \|RAIL7FREQUENCY \|RAIL7PK2PK \|RAIL7RMS \|RAIL7RMSFULL.
        """
        return self._minimum

    @property
    def pk2pk(self) -> MeasuMeas1SubgroupResultsAllacqsPk2pk:
        r"""Return the ``MEASU:MEAS1:SUBGROUP:RESUlts:ALLAcqs:PK2PK`` command.

        Description:
            - This query returns the peak-to-peak value of the measurement specified by the string,
              for all acquisitions.

        Usage:
            - Using the ``.query(argument)`` method will send the
              ``MEASU:MEAS1:SUBGROUP:RESUlts:ALLAcqs:PK2PK? argument`` query.
            - Using the ``.verify(argument, value)`` method will send the
              ``MEASU:MEAS1:SUBGROUP:RESUlts:ALLAcqs:PK2PK? argument`` query and raise an
              AssertionError if the returned value does not match ``value``.

        SCPI Syntax:
            ```
            - MEASU:MEAS1:SUBGROUP:RESUlts:ALLAcqs:PK2PK? <Qstring>
            ```

        Info:
            - ``<Qstring>`` = INPUT\|OUTPUT1\|OUTPUT2\|OUTPUT3\|OUTPUT4\|OUTPUT5\|
              OUTPUT6\|OUTPUT7\| RAIL1DPMOVERSHOOT\|RAIL1DPMUNDERSHOOT \|RAIL1FREQUENCY \|RAIL1PK2PK
              \|RAIL1RMS \|RAIL1RMSFULL\|RAIL2FREQUENCY \|RAIL2PK2PK \|RAIL2RMS \|RAIL2RMSFULL
              \|RAIL3FREQUENCY \|RAIL3PK2PK \|RAIL3RMS\|RAIL3RMSFULL \|RAIL4FREQUENCY \|RAIL4PK2PK
              \|RAIL4RMS \|RAIL4RMSFULL \|RAIL5FREQUENCY \|RAIL5PK2PK\|RAIL5RMS \|RAIL5RMSFULL
              \|RAIL6FREQUENCY \|RAIL6PK2PK \|RAIL6RMS \|RAIL6RMSFULL \|RAIL7DPMOVERSHOOT
              \|RAIL7DPMUNDERSHOOT \|RAIL7FREQUENCY \|RAIL7PK2PK \|RAIL7RMS \|RAIL7RMSFULL.
        """
        return self._pk2pk

    @property
    def population(self) -> MeasuMeas1SubgroupResultsAllacqsPopulation:
        r"""Return the ``MEASU:MEAS1:SUBGROUP:RESUlts:ALLAcqs:POPUlation`` command.

        Description:
            - This query returns the population value of the measurement specified by the string,
              for all acquisitions.

        Usage:
            - Using the ``.query(argument)`` method will send the
              ``MEASU:MEAS1:SUBGROUP:RESUlts:ALLAcqs:POPUlation? argument`` query.
            - Using the ``.verify(argument, value)`` method will send the
              ``MEASU:MEAS1:SUBGROUP:RESUlts:ALLAcqs:POPUlation? argument`` query and raise an
              AssertionError if the returned value does not match ``value``.

        SCPI Syntax:
            ```
            - MEASU:MEAS1:SUBGROUP:RESUlts:ALLAcqs:POPUlation? <Qstring>
            ```

        Info:
            - ``<Qstring>`` = INPUT\|OUTPUT1\|OUTPUT2\|OUTPUT3\|OUTPUT4\|OUTPUT5\|
              OUTPUT6\|OUTPUT7\| RAIL1DPMOVERSHOOT\|RAIL1DPMUNDERSHOOT \|RAIL1FREQUENCY \|RAIL1PK2PK
              \|RAIL1RMS \|RAIL1RMSFULL\|RAIL2FREQUENCY \|RAIL2PK2PK \|RAIL2RMS \|RAIL2RMSFULL
              \|RAIL3FREQUENCY \|RAIL3PK2PK \|RAIL3RMS\|RAIL3RMSFULL \|RAIL4FREQUENCY \|RAIL4PK2PK
              \|RAIL4RMS \|RAIL4RMSFULL \|RAIL5FREQUENCY \|RAIL5PK2PK\|RAIL5RMS \|RAIL5RMSFULL
              \|RAIL6FREQUENCY \|RAIL6PK2PK \|RAIL6RMS \|RAIL6RMSFULL \|RAIL7DPMOVERSHOOT
              \|RAIL7DPMUNDERSHOOT \|RAIL7FREQUENCY \|RAIL7PK2PK \|RAIL7RMS \|RAIL7RMSFULL.
        """
        return self._population

    @property
    def stddev(self) -> MeasuMeas1SubgroupResultsAllacqsStddev:
        r"""Return the ``MEASU:MEAS1:SUBGROUP:RESUlts:ALLAcqs:STDDev`` command.

        Description:
            - This query returns the standard deviation value of the measurement specified by the
              string, for all acquisitions.

        Usage:
            - Using the ``.query(argument)`` method will send the
              ``MEASU:MEAS1:SUBGROUP:RESUlts:ALLAcqs:STDDev? argument`` query.
            - Using the ``.verify(argument, value)`` method will send the
              ``MEASU:MEAS1:SUBGROUP:RESUlts:ALLAcqs:STDDev? argument`` query and raise an
              AssertionError if the returned value does not match ``value``.

        SCPI Syntax:
            ```
            - MEASU:MEAS1:SUBGROUP:RESUlts:ALLAcqs:STDDev? <Qstring>
            ```

        Info:
            - ``<Qstring>`` = INPUT\|OUTPUT1\|OUTPUT2\|OUTPUT3\|OUTPUT4\|OUTPUT5\|
              OUTPUT6\|OUTPUT7\| RAIL1DPMOVERSHOOT\|RAIL1DPMUNDERSHOOT \|RAIL1FREQUENCY \|RAIL1PK2PK
              \|RAIL1RMS \|RAIL1RMSFULL\|RAIL2FREQUENCY \|RAIL2PK2PK \|RAIL2RMS \|RAIL2RMSFULL
              \|RAIL3FREQUENCY \|RAIL3PK2PK \|RAIL3RMS\|RAIL3RMSFULL \|RAIL4FREQUENCY \|RAIL4PK2PK
              \|RAIL4RMS \|RAIL4RMSFULL \|RAIL5FREQUENCY \|RAIL5PK2PK\|RAIL5RMS \|RAIL5RMSFULL
              \|RAIL6FREQUENCY \|RAIL6PK2PK \|RAIL6RMS \|RAIL6RMSFULL \|RAIL7DPMOVERSHOOT
              \|RAIL7DPMUNDERSHOOT \|RAIL7FREQUENCY \|RAIL7PK2PK \|RAIL7RMS \|RAIL7RMSFULL.
        """
        return self._stddev


class MeasuMeas1SubgroupResults(SCPICmdRead):
    """The ``MEASU:MEAS1:SUBGROUP:RESUlts`` command tree.

    Usage:
        - Using the ``.query()`` method will send the ``MEASU:MEAS1:SUBGROUP:RESUlts?`` query.
        - Using the ``.verify(value)`` method will send the ``MEASU:MEAS1:SUBGROUP:RESUlts?`` query
          and raise an AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.allacqs``: The ``MEASU:MEAS1:SUBGROUP:RESUlts:ALLAcqs`` command tree.
        - ``.currentacq``: The ``MEASU:MEAS1:SUBGROUP:RESUlts:CURRentacq`` command tree.
    """

    def __init__(self, device: Optional["PIControl"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._allacqs = MeasuMeas1SubgroupResultsAllacqs(device, f"{self._cmd_syntax}:ALLAcqs")
        self._currentacq = MeasuMeas1SubgroupResultsCurrentacq(
            device, f"{self._cmd_syntax}:CURRentacq"
        )

    @property
    def allacqs(self) -> MeasuMeas1SubgroupResultsAllacqs:
        """Return the ``MEASU:MEAS1:SUBGROUP:RESUlts:ALLAcqs`` command tree.

        Usage:
            - Using the ``.query()`` method will send the ``MEASU:MEAS1:SUBGROUP:RESUlts:ALLAcqs?``
              query.
            - Using the ``.verify(value)`` method will send the
              ``MEASU:MEAS1:SUBGROUP:RESUlts:ALLAcqs?`` query and raise an AssertionError if the
              returned value does not match ``value``.

        Sub-properties:
            - ``.maximum``: The ``MEASU:MEAS1:SUBGROUP:RESUlts:ALLAcqs:MAXimum`` command.
            - ``.mean``: The ``MEASU:MEAS1:SUBGROUP:RESUlts:ALLAcqs:MEAN`` command.
            - ``.minimum``: The ``MEASU:MEAS1:SUBGROUP:RESUlts:ALLAcqs:MINimum`` command.
            - ``.pk2pk``: The ``MEASU:MEAS1:SUBGROUP:RESUlts:ALLAcqs:PK2PK`` command.
            - ``.population``: The ``MEASU:MEAS1:SUBGROUP:RESUlts:ALLAcqs:POPUlation`` command.
            - ``.stddev``: The ``MEASU:MEAS1:SUBGROUP:RESUlts:ALLAcqs:STDDev`` command.
        """
        return self._allacqs

    @property
    def currentacq(self) -> MeasuMeas1SubgroupResultsCurrentacq:
        """Return the ``MEASU:MEAS1:SUBGROUP:RESUlts:CURRentacq`` command tree.

        Usage:
            - Using the ``.query()`` method will send the
              ``MEASU:MEAS1:SUBGROUP:RESUlts:CURRentacq?`` query.
            - Using the ``.verify(value)`` method will send the
              ``MEASU:MEAS1:SUBGROUP:RESUlts:CURRentacq?`` query and raise an AssertionError if the
              returned value does not match ``value``.

        Sub-properties:
            - ``.maximum``: The ``MEASU:MEAS1:SUBGROUP:RESUlts:CURRentacq:MAXimum`` command.
            - ``.mean``: The ``MEASU:MEAS1:SUBGROUP:RESUlts:CURRentacq:MEAN`` command.
            - ``.minimum``: The ``MEASU:MEAS1:SUBGROUP:RESUlts:CURRentacq:MINimum`` command.
            - ``.pk2pk``: The ``MEASU:MEAS1:SUBGROUP:RESUlts:CURRentacq:PK2PK`` command.
            - ``.population``: The ``MEASU:MEAS1:SUBGROUP:RESUlts:CURRentacq:POPUlation`` command.
            - ``.stddev``: The ``MEASU:MEAS1:SUBGROUP:RESUlts:CURRentacq:STDDev`` command.
        """
        return self._currentacq


class MeasuMeas1Subgroup(SCPICmdRead):
    """The ``MEASU:MEAS1:SUBGROUP`` command tree.

    Usage:
        - Using the ``.query()`` method will send the ``MEASU:MEAS1:SUBGROUP?`` query.
        - Using the ``.verify(value)`` method will send the ``MEASU:MEAS1:SUBGROUP?`` query and
          raise an AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.results``: The ``MEASU:MEAS1:SUBGROUP:RESUlts`` command tree.
    """

    def __init__(self, device: Optional["PIControl"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._results = MeasuMeas1SubgroupResults(device, f"{self._cmd_syntax}:RESUlts")

    @property
    def results(self) -> MeasuMeas1SubgroupResults:
        """Return the ``MEASU:MEAS1:SUBGROUP:RESUlts`` command tree.

        Usage:
            - Using the ``.query()`` method will send the ``MEASU:MEAS1:SUBGROUP:RESUlts?`` query.
            - Using the ``.verify(value)`` method will send the ``MEASU:MEAS1:SUBGROUP:RESUlts?``
              query and raise an AssertionError if the returned value does not match ``value``.

        Sub-properties:
            - ``.allacqs``: The ``MEASU:MEAS1:SUBGROUP:RESUlts:ALLAcqs`` command tree.
            - ``.currentacq``: The ``MEASU:MEAS1:SUBGROUP:RESUlts:CURRentacq`` command tree.
        """
        return self._results


class MeasuMeas1(SCPICmdRead):
    """The ``MEASU:MEAS1`` command tree.

    Usage:
        - Using the ``.query()`` method will send the ``MEASU:MEAS1?`` query.
        - Using the ``.verify(value)`` method will send the ``MEASU:MEAS1?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.subgroup``: The ``MEASU:MEAS1:SUBGROUP`` command tree.
    """

    def __init__(self, device: Optional["PIControl"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._subgroup = MeasuMeas1Subgroup(device, f"{self._cmd_syntax}:SUBGROUP")

    @property
    def subgroup(self) -> MeasuMeas1Subgroup:
        """Return the ``MEASU:MEAS1:SUBGROUP`` command tree.

        Usage:
            - Using the ``.query()`` method will send the ``MEASU:MEAS1:SUBGROUP?`` query.
            - Using the ``.verify(value)`` method will send the ``MEASU:MEAS1:SUBGROUP?`` query and
              raise an AssertionError if the returned value does not match ``value``.

        Sub-properties:
            - ``.results``: The ``MEASU:MEAS1:SUBGROUP:RESUlts`` command tree.
        """
        return self._subgroup


class Measu(SCPICmdRead):
    """The ``MEASU`` command tree.

    Usage:
        - Using the ``.query()`` method will send the ``MEASU?`` query.
        - Using the ``.verify(value)`` method will send the ``MEASU?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.meas1``: The ``MEASU:MEAS1`` command tree.
    """

    def __init__(self, device: Optional["PIControl"] = None, cmd_syntax: str = "MEASU") -> None:
        super().__init__(device, cmd_syntax)
        self._meas1 = MeasuMeas1(device, f"{self._cmd_syntax}:MEAS1")

    @property
    def meas1(self) -> MeasuMeas1:
        """Return the ``MEASU:MEAS1`` command tree.

        Usage:
            - Using the ``.query()`` method will send the ``MEASU:MEAS1?`` query.
            - Using the ``.verify(value)`` method will send the ``MEASU:MEAS1?`` query and raise an
              AssertionError if the returned value does not match ``value``.

        Sub-properties:
            - ``.subgroup``: The ``MEASU:MEAS1:SUBGROUP`` command tree.
        """
        return self._meas1



"""The d commands module.

These commands are used in the following models:
DPO5K, DPO5KB, DPO70KC, DPO70KD, DPO70KDX, DPO70KSX, DPO7K, DPO7KC, DSA70KC, DSA70KD, MSO5K, MSO5KB,
MSO70KC, MSO70KDX

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Commands and Queries:
    ```
    - D<x>:LABEL <QString>
    - D<x>:LABEL?
    - D<x>:POSition <NR3>
    - D<x>:POSition?
    - D<x>:PROBE:ID:SERnumber?
    - D<x>:PROBE:ID:TYPe?
    - D<x>:THRESHold <NR3>
    - D<x>:THRESHold?
    ```
"""

from typing import Optional, TYPE_CHECKING

from ..helpers import SCPICmdRead, SCPICmdWrite, ValidatedDigitalBit

if TYPE_CHECKING:
    from tm_devices.driver_mixins.device_control.pi_control import PIControl


class DigitalBitThreshold(SCPICmdWrite, SCPICmdRead):
    """The ``D<x>:THRESHold`` command.

    Description:
        - This command sets or queries the threshold level for the specified digital signal.

    Usage:
        - Using the ``.query()`` method will send the ``D<x>:THRESHold?`` query.
        - Using the ``.verify(value)`` method will send the ``D<x>:THRESHold?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``D<x>:THRESHold value`` command.

    SCPI Syntax:
        ```
        - D<x>:THRESHold <NR3>
        - D<x>:THRESHold?
        ```

    Info:
        - ``<NR3>`` specifies the threshold level in volts.
    """


class DigitalBitProbeIdType(SCPICmdRead):
    """The ``D<x>:PROBE:ID:TYPe`` command.

    Description:
        - This command queries the type of digital probe that provides the specified digital signal.

    Usage:
        - Using the ``.query()`` method will send the ``D<x>:PROBE:ID:TYPe?`` query.
        - Using the ``.verify(value)`` method will send the ``D<x>:PROBE:ID:TYPe?`` query and raise
          an AssertionError if the returned value does not match ``value``.

    SCPI Syntax:
        ```
        - D<x>:PROBE:ID:TYPe?
        ```
    """


class DigitalBitProbeIdSernumber(SCPICmdRead):
    """The ``D<x>:PROBE:ID:SERnumber`` command.

    Description:
        - This command queries the serial number of the digital probe that provides the specified
          digital signal.

    Usage:
        - Using the ``.query()`` method will send the ``D<x>:PROBE:ID:SERnumber?`` query.
        - Using the ``.verify(value)`` method will send the ``D<x>:PROBE:ID:SERnumber?`` query and
          raise an AssertionError if the returned value does not match ``value``.

    SCPI Syntax:
        ```
        - D<x>:PROBE:ID:SERnumber?
        ```
    """


class DigitalBitProbeId(SCPICmdRead):
    """The ``D<x>:PROBE:ID`` command tree.

    Usage:
        - Using the ``.query()`` method will send the ``D<x>:PROBE:ID?`` query.
        - Using the ``.verify(value)`` method will send the ``D<x>:PROBE:ID?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.sernumber``: The ``D<x>:PROBE:ID:SERnumber`` command.
        - ``.type``: The ``D<x>:PROBE:ID:TYPe`` command.
    """

    def __init__(self, device: Optional["PIControl"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._sernumber = DigitalBitProbeIdSernumber(device, f"{self._cmd_syntax}:SERnumber")
        self._type = DigitalBitProbeIdType(device, f"{self._cmd_syntax}:TYPe")

    @property
    def sernumber(self) -> DigitalBitProbeIdSernumber:
        """Return the ``D<x>:PROBE:ID:SERnumber`` command.

        Description:
            - This command queries the serial number of the digital probe that provides the
              specified digital signal.

        Usage:
            - Using the ``.query()`` method will send the ``D<x>:PROBE:ID:SERnumber?`` query.
            - Using the ``.verify(value)`` method will send the ``D<x>:PROBE:ID:SERnumber?`` query
              and raise an AssertionError if the returned value does not match ``value``.

        SCPI Syntax:
            ```
            - D<x>:PROBE:ID:SERnumber?
            ```
        """
        return self._sernumber

    @property
    def type(self) -> DigitalBitProbeIdType:
        """Return the ``D<x>:PROBE:ID:TYPe`` command.

        Description:
            - This command queries the type of digital probe that provides the specified digital
              signal.

        Usage:
            - Using the ``.query()`` method will send the ``D<x>:PROBE:ID:TYPe?`` query.
            - Using the ``.verify(value)`` method will send the ``D<x>:PROBE:ID:TYPe?`` query and
              raise an AssertionError if the returned value does not match ``value``.

        SCPI Syntax:
            ```
            - D<x>:PROBE:ID:TYPe?
            ```
        """
        return self._type


class DigitalBitProbe(SCPICmdRead):
    """The ``D<x>:PROBE`` command tree.

    Usage:
        - Using the ``.query()`` method will send the ``D<x>:PROBE?`` query.
        - Using the ``.verify(value)`` method will send the ``D<x>:PROBE?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.id``: The ``D<x>:PROBE:ID`` command tree.
    """

    def __init__(self, device: Optional["PIControl"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._id = DigitalBitProbeId(device, f"{self._cmd_syntax}:ID")

    @property
    def id(self) -> DigitalBitProbeId:
        """Return the ``D<x>:PROBE:ID`` command tree.

        Usage:
            - Using the ``.query()`` method will send the ``D<x>:PROBE:ID?`` query.
            - Using the ``.verify(value)`` method will send the ``D<x>:PROBE:ID?`` query and raise
              an AssertionError if the returned value does not match ``value``.

        Sub-properties:
            - ``.sernumber``: The ``D<x>:PROBE:ID:SERnumber`` command.
            - ``.type``: The ``D<x>:PROBE:ID:TYPe`` command.
        """
        return self._id


class DigitalBitPosition(SCPICmdWrite, SCPICmdRead):
    """The ``D<x>:POSition`` command.

    Description:
        - This command specifies the vertical position for digital channel <x>, where x is the
          channel number.

    Usage:
        - Using the ``.query()`` method will send the ``D<x>:POSition?`` query.
        - Using the ``.verify(value)`` method will send the ``D<x>:POSition?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``D<x>:POSition value`` command.

    SCPI Syntax:
        ```
        - D<x>:POSition <NR3>
        - D<x>:POSition?
        ```

    Info:
        - ``<NR3>`` is a floating point number that specifies the position of the digital channel,
          in slot units. Use the oscilloscope front-panel controls to place the channel; then query
          the channel to obtain an exact value for the position.
    """


class DigitalBitLabel(SCPICmdWrite, SCPICmdRead):
    """The ``D<x>:LABEL`` command.

    Description:
        - This command sets or queries the label that appears for the specified digital input on the
          display.

    Usage:
        - Using the ``.query()`` method will send the ``D<x>:LABEL?`` query.
        - Using the ``.verify(value)`` method will send the ``D<x>:LABEL?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``D<x>:LABEL value`` command.

    SCPI Syntax:
        ```
        - D<x>:LABEL <QString>
        - D<x>:LABEL?
        ```

    Info:
        - ``<QString>`` is an alphanumeric string of characters, enclosed in quotes, that defines
          the label text.
    """

    _WRAP_ARG_WITH_QUOTES = True


class DigitalBit(ValidatedDigitalBit, SCPICmdRead):
    """The ``D<x>`` command tree.

    Usage:
        - Using the ``.query()`` method will send the ``D<x>?`` query.
        - Using the ``.verify(value)`` method will send the ``D<x>?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.label``: The ``D<x>:LABEL`` command.
        - ``.position``: The ``D<x>:POSition`` command.
        - ``.probe``: The ``D<x>:PROBE`` command tree.
        - ``.threshold``: The ``D<x>:THRESHold`` command.
    """

    def __init__(self, device: Optional["PIControl"] = None, cmd_syntax: str = "D<x>") -> None:
        super().__init__(device, cmd_syntax)
        self._label = DigitalBitLabel(device, f"{self._cmd_syntax}:LABEL")
        self._position = DigitalBitPosition(device, f"{self._cmd_syntax}:POSition")
        self._probe = DigitalBitProbe(device, f"{self._cmd_syntax}:PROBE")
        self._threshold = DigitalBitThreshold(device, f"{self._cmd_syntax}:THRESHold")

    @property
    def label(self) -> DigitalBitLabel:
        """Return the ``D<x>:LABEL`` command.

        Description:
            - This command sets or queries the label that appears for the specified digital input on
              the display.

        Usage:
            - Using the ``.query()`` method will send the ``D<x>:LABEL?`` query.
            - Using the ``.verify(value)`` method will send the ``D<x>:LABEL?`` query and raise an
              AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``D<x>:LABEL value`` command.

        SCPI Syntax:
            ```
            - D<x>:LABEL <QString>
            - D<x>:LABEL?
            ```

        Info:
            - ``<QString>`` is an alphanumeric string of characters, enclosed in quotes, that
              defines the label text.
        """
        return self._label

    @property
    def position(self) -> DigitalBitPosition:
        """Return the ``D<x>:POSition`` command.

        Description:
            - This command specifies the vertical position for digital channel <x>, where x is the
              channel number.

        Usage:
            - Using the ``.query()`` method will send the ``D<x>:POSition?`` query.
            - Using the ``.verify(value)`` method will send the ``D<x>:POSition?`` query and raise
              an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``D<x>:POSition value`` command.

        SCPI Syntax:
            ```
            - D<x>:POSition <NR3>
            - D<x>:POSition?
            ```

        Info:
            - ``<NR3>`` is a floating point number that specifies the position of the digital
              channel, in slot units. Use the oscilloscope front-panel controls to place the
              channel; then query the channel to obtain an exact value for the position.
        """
        return self._position

    @property
    def probe(self) -> DigitalBitProbe:
        """Return the ``D<x>:PROBE`` command tree.

        Usage:
            - Using the ``.query()`` method will send the ``D<x>:PROBE?`` query.
            - Using the ``.verify(value)`` method will send the ``D<x>:PROBE?`` query and raise an
              AssertionError if the returned value does not match ``value``.

        Sub-properties:
            - ``.id``: The ``D<x>:PROBE:ID`` command tree.
        """
        return self._probe

    @property
    def threshold(self) -> DigitalBitThreshold:
        """Return the ``D<x>:THRESHold`` command.

        Description:
            - This command sets or queries the threshold level for the specified digital signal.

        Usage:
            - Using the ``.query()`` method will send the ``D<x>:THRESHold?`` query.
            - Using the ``.verify(value)`` method will send the ``D<x>:THRESHold?`` query and raise
              an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``D<x>:THRESHold value`` command.

        SCPI Syntax:
            ```
            - D<x>:THRESHold <NR3>
            - D<x>:THRESHold?
            ```

        Info:
            - ``<NR3>`` specifies the threshold level in volts.
        """
        return self._threshold



"""The clear commands module.

These commands are used in the following models:
DPO7AX, LPD6, MDO3, MSO2, MSO4, MSO4B, MSO5, MSO5B, MSO5LP, MSO6, MSO6B

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Commands and Queries:
    ```
    - CLEAR
    ```
"""

from typing import Optional, TYPE_CHECKING

from ..helpers import SCPICmdWriteNoArguments

if TYPE_CHECKING:
    from tm_devices.driver_mixins.device_control.pi_control import PIControl


class Clear(SCPICmdWriteNoArguments):
    """The ``CLEAR`` command.

    Description:
        - This command clears acquisitions, measurements, and waveforms.

    Usage:
        - Using the ``.write()`` method will send the ``CLEAR`` command.

    SCPI Syntax:
        ```
        - CLEAR
        ```
    """

    def __init__(self, device: Optional["PIControl"] = None, cmd_syntax: str = "CLEAR") -> None:
        super().__init__(device, cmd_syntax)



"""The auxout commands module.

These commands are used in the following models:
DPO5K, DPO5KB, DPO70KC, DPO70KD, DPO70KDX, DPO70KSX, DPO7K, DPO7KC, DSA70KC, DSA70KD, MSO5K, MSO5KB,
MSO70KC, MSO70KDX

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Commands and Queries:
    ```
    - AUXout:EDGE {RISing|FALling}
    - AUXout:EDGE?
    - AUXout:SOUrce {ATRIGger|BTRIGger|DELayed|EVENT|REFOUT}
    - AUXout:SOUrce?
    - AUXout?
    ```
"""

from typing import Optional, TYPE_CHECKING

from ..helpers import SCPICmdRead, SCPICmdWrite

if TYPE_CHECKING:
    from tm_devices.driver_mixins.device_control.pi_control import PIControl


class AuxoutSource(SCPICmdWrite, SCPICmdRead):
    """The ``AUXout:SOUrce`` command.

    Description:
        - This command sets or queries the trigger source at the BNC connection. This command is
          equivalent to selecting External Signals from the Utilities menu and then selecting the
          desired Configuration setting.

    Usage:
        - Using the ``.query()`` method will send the ``AUXout:SOUrce?`` query.
        - Using the ``.verify(value)`` method will send the ``AUXout:SOUrce?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``AUXout:SOUrce value`` command.

    SCPI Syntax:
        ```
        - AUXout:SOUrce {ATRIGger|BTRIGger|DELayed|EVENT|REFOUT}
        - AUXout:SOUrce?
        ```

    Info:
        - ``ATRIGger`` sets the source at the BNC connector to the main trigger.
        - ``BTRIGger`` sets the source at the BNC connector to the delayed trigger. (7K/70K).
        - ``EVENT`` sets the source at the BNC connector to a specified event. (7K/70K).
        - ``DELayed`` sets the source at the BNC connector to the delayed trigger.
        - ``REFOUT`` sets the source at the BNC connector to the reference output. (5K/7K).
    """


class AuxoutEdge(SCPICmdWrite, SCPICmdRead):
    """The ``AUXout:EDGE`` command.

    Description:
        - This command sets or queries the direction in which the Auxiliary Output signal will
          transition when a trigger occurs.

    Usage:
        - Using the ``.query()`` method will send the ``AUXout:EDGE?`` query.
        - Using the ``.verify(value)`` method will send the ``AUXout:EDGE?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``AUXout:EDGE value`` command.

    SCPI Syntax:
        ```
        - AUXout:EDGE {RISing|FALling}
        - AUXout:EDGE?
        ```

    Info:
        - ``RISing`` sets the polarity to the rising edge.
        - ``FALling`` sets the polarity to the falling edge.
    """


class Auxout(SCPICmdRead):
    """The ``AUXout`` command.

    Description:
        - This query-only command returns the auxiliary output setup and is equivalent to selecting
          External Signals. From the Utilities menu, and then viewing the current settings for the
          AUX OUT Configuration.

    Usage:
        - Using the ``.query()`` method will send the ``AUXout?`` query.
        - Using the ``.verify(value)`` method will send the ``AUXout?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    SCPI Syntax:
        ```
        - AUXout?
        ```

    Properties:
        - ``.edge``: The ``AUXout:EDGE`` command.
        - ``.source``: The ``AUXout:SOUrce`` command.
    """

    def __init__(self, device: Optional["PIControl"] = None, cmd_syntax: str = "AUXout") -> None:
        super().__init__(device, cmd_syntax)
        self._edge = AuxoutEdge(device, f"{self._cmd_syntax}:EDGE")
        self._source = AuxoutSource(device, f"{self._cmd_syntax}:SOUrce")

    @property
    def edge(self) -> AuxoutEdge:
        """Return the ``AUXout:EDGE`` command.

        Description:
            - This command sets or queries the direction in which the Auxiliary Output signal will
              transition when a trigger occurs.

        Usage:
            - Using the ``.query()`` method will send the ``AUXout:EDGE?`` query.
            - Using the ``.verify(value)`` method will send the ``AUXout:EDGE?`` query and raise an
              AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``AUXout:EDGE value`` command.

        SCPI Syntax:
            ```
            - AUXout:EDGE {RISing|FALling}
            - AUXout:EDGE?
            ```

        Info:
            - ``RISing`` sets the polarity to the rising edge.
            - ``FALling`` sets the polarity to the falling edge.
        """
        return self._edge

    @property
    def source(self) -> AuxoutSource:
        """Return the ``AUXout:SOUrce`` command.

        Description:
            - This command sets or queries the trigger source at the BNC connection. This command is
              equivalent to selecting External Signals from the Utilities menu and then selecting
              the desired Configuration setting.

        Usage:
            - Using the ``.query()`` method will send the ``AUXout:SOUrce?`` query.
            - Using the ``.verify(value)`` method will send the ``AUXout:SOUrce?`` query and raise
              an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``AUXout:SOUrce value`` command.

        SCPI Syntax:
            ```
            - AUXout:SOUrce {ATRIGger|BTRIGger|DELayed|EVENT|REFOUT}
            - AUXout:SOUrce?
            ```

        Info:
            - ``ATRIGger`` sets the source at the BNC connector to the main trigger.
            - ``BTRIGger`` sets the source at the BNC connector to the delayed trigger. (7K/70K).
            - ``EVENT`` sets the source at the BNC connector to a specified event. (7K/70K).
            - ``DELayed`` sets the source at the BNC connector to the delayed trigger.
            - ``REFOUT`` sets the source at the BNC connector to the reference output. (5K/7K).
        """
        return self._source



"""AFG31K device driver module."""

from typing import Optional, Tuple

from tm_devices.drivers.afgs.afg3k import (
    AFG,
    AFGSourceDeviceConstants,
    LoadImpedanceAFG,
    ParameterBounds,
    SignalGeneratorFunctionsAFG,
)


class AFG31K(AFG):
    """AFG31K device driver."""

    _DEVICE_CONSTANTS = AFGSourceDeviceConstants(
        memory_page_size=2,
        memory_max_record_length=131072,
        memory_min_record_length=2,
    )

    _PRE_15_FREQ_THRESHOLD_1 = 60.0e6
    _PRE_15_FREQ_THRESHOLD_2 = 80.0e6
    _POST_15_FREQ_THRESHOLD = 200.0e6
    _16KB_THRESHOLD = 16 * 1024

    ################################################################################################
    # Magic Methods
    ################################################################################################

    ################################################################################################
    # Properties
    ################################################################################################

    ################################################################################################
    # Public Methods
    ################################################################################################

    ################################################################################################
    # Private Methods
    ################################################################################################
    @staticmethod
    def _get_driver_specific_multipliers(model_number: str) -> Tuple[float, float, float]:
        """Get multipliers for frequency dependent on the function.

        Args:
            model_number: The numbers not pertaining to series or channel count in the name.
                ie: AFG31 ->25<- 1

        Returns:
            The necessary values to multiply the frequency by dependent on the function.
        """
        # the square wave constraints are difference for model 25
        square_wave_multiplier = 0.64 if model_number == "25" else 0.8

        # other wave constraints are 1% of the SIN wave constraints, except for models 02 and 05
        if model_number == "02":
            other_wave_multiplier = 0.025
        elif model_number == "05":
            other_wave_multiplier = 0.016
        else:
            other_wave_multiplier = 0.01

        # the arbitrary waveform constraints are half og the SIN wave constraints
        arbitrary_multiplier = 0.5
        return square_wave_multiplier, other_wave_multiplier, arbitrary_multiplier

    # pylint: disable=too-many-locals
    def _get_series_specific_constraints(
        self,
        function: SignalGeneratorFunctionsAFG,
        waveform_length: Optional[int] = None,
        frequency: Optional[float] = None,
        load_impedance: LoadImpedanceAFG = LoadImpedanceAFG.HIGHZ,
    ) -> Tuple[ParameterBounds, ParameterBounds, ParameterBounds, ParameterBounds]:
        """Get constraints which are dependent on the model series and parameters.

        Args:
            function: The function that needs to be generated.
            waveform_length: The length of the waveform if no function or arbitrary is provided.
            frequency: The frequency of the waveform that needs to be generated.
            load_impedance: The suggested impedance on the source.

        Returns:
            Ranges for amplitude, frequency, offset, and sample rate.
        """
        # the model number is the third and fourth digit of the model serial, ex. 31(25)2
        model_number = self.model[5:7]

        load_impedance_multiplier = 1.0 if load_impedance == LoadImpedanceAFG.HIGHZ else 0.5

        freq_base = 10.0e6

        lower_amplitude = 2.0e-3
        # the higher the frequency, the lower the maximum amplitude can be
        if model_number in {"02", "05", "10"}:
            if frequency and frequency <= self._PRE_15_FREQ_THRESHOLD_1:
                upper_amplitude = 20.0
            elif frequency and frequency <= self._PRE_15_FREQ_THRESHOLD_2:
                upper_amplitude = 16.0
            else:
                upper_amplitude = 12.0
            # offset bound is always 10.0
            offset_bound = 10.0
        else:
            upper_amplitude = (
                10.0 if (frequency and frequency <= self._POST_15_FREQ_THRESHOLD) else 8.0
            )
            # offset bound is always 5.0
            offset_bound = 5.0

        # special case for 02
        model_multiplier = 2.5 if "02" in model_number else float(model_number)

        sample_rate = 250.0e6
        # if the waveform length is less than 16 KB, than the AFG can use a higher sample rate
        if waveform_length and waveform_length < self._16KB_THRESHOLD:
            if model_number in {"05", "10"}:
                sample_rate = 1.0e9
            elif model_number in {"15", "25"}:
                sample_rate = 2.0e9

        square_mult, other_mult, arb_mult = self._get_driver_specific_multipliers(model_number)
        low_freq_range = 1.0e-6
        # the maximum frequency of the SIN wave depends on the model mumber
        if function.name in {SignalGeneratorFunctionsAFG.SIN.name}:
            high_freq_range = model_multiplier * freq_base
        # each waveform has a maximum frequency which is a multiple of SIN wave constraints
        elif function.name == SignalGeneratorFunctionsAFG.ARBITRARY.name:
            high_freq_range = model_multiplier * arb_mult * freq_base
        elif function.name in {
            SignalGeneratorFunctionsAFG.PULSE.name,
            SignalGeneratorFunctionsAFG.SQUARE.name,
        }:
            high_freq_range = model_multiplier * square_mult * freq_base
        else:
            high_freq_range = model_multiplier * other_mult * freq_base

        # load impedance can double the amplitude and offset range
        amplitude_range = ParameterBounds(
            lower=lower_amplitude * load_impedance_multiplier,
            upper=upper_amplitude * load_impedance_multiplier,
        )

        frequency_range = ParameterBounds(
            lower=low_freq_range,
            upper=high_freq_range,
        )

        offset_range = ParameterBounds(
            lower=-offset_bound * load_impedance_multiplier,
            upper=offset_bound * load_impedance_multiplier,
        )

        sample_rate_range = ParameterBounds(lower=sample_rate, upper=sample_rate)

        return amplitude_range, frequency_range, offset_range, sample_rate_range

    def _reboot(self) -> None:
        """Reboot the device."""
        self.write("SYSTem:RESTart")



"""The diag commands module.

These commands are used in the following models:
DPO7AX

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Commands and Queries:
    ```
    - DIAg:LOOP:OPTion {FAIL|ONCE|ALWAYS|ONFAIL|NTIMES}
    - DIAg:LOOP:OPTion:NTIMes <NR1>
    - DIAg:LOOP:OPTion:NTIMes?
    - DIAg:LOOP:OPTion?
    - DIAg:LOOP:STOP
    - DIAg:MODe {POST|EXTENDED}
    - DIAg:MODe?
    - DIAg:RESUlt:FLAg?
    - DIAg:RESUlt:LOG?
    - DIAg:RESUlt?
    - DIAg:SELect {ALL|IO|ANALOG|SYSTEM|ASIC|ACQ|SIGNAL|MEMORY}
    - DIAg:STATE {EXECute|ABOrt}
    ```
"""

from typing import Optional, TYPE_CHECKING

from ..helpers import SCPICmdRead, SCPICmdWrite, SCPICmdWriteNoArguments

if TYPE_CHECKING:
    from tm_devices.driver_mixins.device_control.pi_control import PIControl


class DiagState(SCPICmdWrite):
    """The ``DIAg:STATE`` command.

    Description:
        - This command starts or aborts Self Test. Abort happens after group under test completes.

    Usage:
        - Using the ``.write(value)`` method will send the ``DIAg:STATE value`` command.

    SCPI Syntax:
        ```
        - DIAg:STATE {EXECute|ABOrt}
        ```

    Info:
        - ``EXECute`` starts execution of the diagnostics.
        - ``ABOrt`` disables diagnostics capabilities and returns the instrument to a normal
          operating state.
    """


class DiagSelect(SCPICmdWrite):
    """The ``DIAg:SELect`` command.

    Description:
        - This command selects an available diagnostic area.

    Usage:
        - Using the ``.write(value)`` method will send the ``DIAg:SELect value`` command.

    SCPI Syntax:
        ```
        - DIAg:SELect {ALL|IO|ANALOG|SYSTEM|ASIC|ACQ|SIGNAL|MEMORY}
        ```

    Info:
        - ``ALL`` selects all diagnostic areas.
        - ``IO`` selects the IO group.
        - ``ANALOG`` selects the ANALOG diagnostic area.
        - ``SYSTEM`` selects the SYSTEM diagnostic area.
        - ``ASIC`` selects the ASIC diagnostic area.
        - ``ACQ`` selects the ACQ diagnostic area.
        - ``SIGNAL`` selects the SIGNAL diagnostic area.
        - ``MEMORY`` selects the MEMORY diagnostic area.
    """


class DiagResultLog(SCPICmdRead):
    """The ``DIAg:RESUlt:LOG`` command.

    Description:
        - This query returns the test Pass/Fail status of each diagnostic area. It does not return
          the overall status.

    Usage:
        - Using the ``.query()`` method will send the ``DIAg:RESUlt:LOG?`` query.
        - Using the ``.verify(value)`` method will send the ``DIAg:RESUlt:LOG?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    SCPI Syntax:
        ```
        - DIAg:RESUlt:LOG?
        ```
    """


class DiagResultFlag(SCPICmdRead):
    """The ``DIAg:RESUlt:FLAg`` command.

    Description:
        - This query returns the status of the diagnostic test area that has been selected.

    Usage:
        - Using the ``.query()`` method will send the ``DIAg:RESUlt:FLAg?`` query.
        - Using the ``.verify(value)`` method will send the ``DIAg:RESUlt:FLAg?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    SCPI Syntax:
        ```
        - DIAg:RESUlt:FLAg?
        ```
    """


class DiagResult(SCPICmdRead):
    """The ``DIAg:RESUlt`` command.

    Description:
        - This query returns both the overall diagnostics test results and the results of each
          individual test area.

    Usage:
        - Using the ``.query()`` method will send the ``DIAg:RESUlt?`` query.
        - Using the ``.verify(value)`` method will send the ``DIAg:RESUlt?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    SCPI Syntax:
        ```
        - DIAg:RESUlt?
        ```

    Properties:
        - ``.flag``: The ``DIAg:RESUlt:FLAg`` command.
        - ``.log``: The ``DIAg:RESUlt:LOG`` command.
    """

    def __init__(self, device: Optional["PIControl"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._flag = DiagResultFlag(device, f"{self._cmd_syntax}:FLAg")
        self._log = DiagResultLog(device, f"{self._cmd_syntax}:LOG")

    @property
    def flag(self) -> DiagResultFlag:
        """Return the ``DIAg:RESUlt:FLAg`` command.

        Description:
            - This query returns the status of the diagnostic test area that has been selected.

        Usage:
            - Using the ``.query()`` method will send the ``DIAg:RESUlt:FLAg?`` query.
            - Using the ``.verify(value)`` method will send the ``DIAg:RESUlt:FLAg?`` query and
              raise an AssertionError if the returned value does not match ``value``.

        SCPI Syntax:
            ```
            - DIAg:RESUlt:FLAg?
            ```
        """
        return self._flag

    @property
    def log(self) -> DiagResultLog:
        """Return the ``DIAg:RESUlt:LOG`` command.

        Description:
            - This query returns the test Pass/Fail status of each diagnostic area. It does not
              return the overall status.

        Usage:
            - Using the ``.query()`` method will send the ``DIAg:RESUlt:LOG?`` query.
            - Using the ``.verify(value)`` method will send the ``DIAg:RESUlt:LOG?`` query and raise
              an AssertionError if the returned value does not match ``value``.

        SCPI Syntax:
            ```
            - DIAg:RESUlt:LOG?
            ```
        """
        return self._log


class DiagMode(SCPICmdWrite, SCPICmdRead):
    """The ``DIAg:MODe`` command.

    Description:
        - This command sets or queries the diagnostics mode.

    Usage:
        - Using the ``.query()`` method will send the ``DIAg:MODe?`` query.
        - Using the ``.verify(value)`` method will send the ``DIAg:MODe?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``DIAg:MODe value`` command.

    SCPI Syntax:
        ```
        - DIAg:MODe {POST|EXTENDED}
        - DIAg:MODe?
        ```

    Info:
        - ``POST`` specifies the power on self test diagnostics.
        - ``EXTENDED`` specifies the extended diagnostics.
    """


class DiagLoopStop(SCPICmdWriteNoArguments):
    """The ``DIAg:LOOP:STOP`` command.

    Description:
        - Request that diagnostics stop looping.

    Usage:
        - Using the ``.write()`` method will send the ``DIAg:LOOP:STOP`` command.

    SCPI Syntax:
        ```
        - DIAg:LOOP:STOP
        ```
    """


class DiagLoopOptionNtimes(SCPICmdWrite, SCPICmdRead):
    """The ``DIAg:LOOP:OPTion:NTIMes`` command.

    Description:
        - This command sets or queries how many loops to run, if N-times is being used.

    Usage:
        - Using the ``.query()`` method will send the ``DIAg:LOOP:OPTion:NTIMes?`` query.
        - Using the ``.verify(value)`` method will send the ``DIAg:LOOP:OPTion:NTIMes?`` query and
          raise an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``DIAg:LOOP:OPTion:NTIMes value``
          command.

    SCPI Syntax:
        ```
        - DIAg:LOOP:OPTion:NTIMes <NR1>
        - DIAg:LOOP:OPTion:NTIMes?
        ```

    Info:
        - ``<NR1>`` is how many loops to run.
    """


class DiagLoopOption(SCPICmdWrite, SCPICmdRead):
    """The ``DIAg:LOOP:OPTion`` command.

    Description:
        - This command sets or queries the type of looping desired.

    Usage:
        - Using the ``.query()`` method will send the ``DIAg:LOOP:OPTion?`` query.
        - Using the ``.verify(value)`` method will send the ``DIAg:LOOP:OPTion?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``DIAg:LOOP:OPTion value`` command.

    SCPI Syntax:
        ```
        - DIAg:LOOP:OPTion {FAIL|ONCE|ALWAYS|ONFAIL|NTIMES}
        - DIAg:LOOP:OPTion?
        ```

    Info:
        - ``FAIL`` - run until a failure is found, then halt.
        - ``ONCE`` - run through one loop.
        - ``ALWAYS`` - run forever.
        - ``ONFAIL`` - run until a failure is found, then loop on it.
        - ``NTIMES`` - run n number of loops.

    Properties:
        - ``.ntimes``: The ``DIAg:LOOP:OPTion:NTIMes`` command.
    """

    def __init__(self, device: Optional["PIControl"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._ntimes = DiagLoopOptionNtimes(device, f"{self._cmd_syntax}:NTIMes")

    @property
    def ntimes(self) -> DiagLoopOptionNtimes:
        """Return the ``DIAg:LOOP:OPTion:NTIMes`` command.

        Description:
            - This command sets or queries how many loops to run, if N-times is being used.

        Usage:
            - Using the ``.query()`` method will send the ``DIAg:LOOP:OPTion:NTIMes?`` query.
            - Using the ``.verify(value)`` method will send the ``DIAg:LOOP:OPTion:NTIMes?`` query
              and raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``DIAg:LOOP:OPTion:NTIMes value``
              command.

        SCPI Syntax:
            ```
            - DIAg:LOOP:OPTion:NTIMes <NR1>
            - DIAg:LOOP:OPTion:NTIMes?
            ```

        Info:
            - ``<NR1>`` is how many loops to run.
        """
        return self._ntimes


class DiagLoop(SCPICmdRead):
    """The ``DIAg:LOOP`` command tree.

    Usage:
        - Using the ``.query()`` method will send the ``DIAg:LOOP?`` query.
        - Using the ``.verify(value)`` method will send the ``DIAg:LOOP?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.option``: The ``DIAg:LOOP:OPTion`` command.
        - ``.stop``: The ``DIAg:LOOP:STOP`` command.
    """

    def __init__(self, device: Optional["PIControl"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._option = DiagLoopOption(device, f"{self._cmd_syntax}:OPTion")
        self._stop = DiagLoopStop(device, f"{self._cmd_syntax}:STOP")

    @property
    def option(self) -> DiagLoopOption:
        """Return the ``DIAg:LOOP:OPTion`` command.

        Description:
            - This command sets or queries the type of looping desired.

        Usage:
            - Using the ``.query()`` method will send the ``DIAg:LOOP:OPTion?`` query.
            - Using the ``.verify(value)`` method will send the ``DIAg:LOOP:OPTion?`` query and
              raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``DIAg:LOOP:OPTion value`` command.

        SCPI Syntax:
            ```
            - DIAg:LOOP:OPTion {FAIL|ONCE|ALWAYS|ONFAIL|NTIMES}
            - DIAg:LOOP:OPTion?
            ```

        Info:
            - ``FAIL`` - run until a failure is found, then halt.
            - ``ONCE`` - run through one loop.
            - ``ALWAYS`` - run forever.
            - ``ONFAIL`` - run until a failure is found, then loop on it.
            - ``NTIMES`` - run n number of loops.

        Sub-properties:
            - ``.ntimes``: The ``DIAg:LOOP:OPTion:NTIMes`` command.
        """
        return self._option

    @property
    def stop(self) -> DiagLoopStop:
        """Return the ``DIAg:LOOP:STOP`` command.

        Description:
            - Request that diagnostics stop looping.

        Usage:
            - Using the ``.write()`` method will send the ``DIAg:LOOP:STOP`` command.

        SCPI Syntax:
            ```
            - DIAg:LOOP:STOP
            ```
        """
        return self._stop


class Diag(SCPICmdRead):
    """The ``DIAg`` command tree.

    Usage:
        - Using the ``.query()`` method will send the ``DIAg?`` query.
        - Using the ``.verify(value)`` method will send the ``DIAg?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.loop``: The ``DIAg:LOOP`` command tree.
        - ``.mode``: The ``DIAg:MODe`` command.
        - ``.result``: The ``DIAg:RESUlt`` command.
        - ``.select``: The ``DIAg:SELect`` command.
        - ``.state``: The ``DIAg:STATE`` command.
    """

    def __init__(self, device: Optional["PIControl"] = None, cmd_syntax: str = "DIAg") -> None:
        super().__init__(device, cmd_syntax)
        self._loop = DiagLoop(device, f"{self._cmd_syntax}:LOOP")
        self._mode = DiagMode(device, f"{self._cmd_syntax}:MODe")
        self._result = DiagResult(device, f"{self._cmd_syntax}:RESUlt")
        self._select = DiagSelect(device, f"{self._cmd_syntax}:SELect")
        self._state = DiagState(device, f"{self._cmd_syntax}:STATE")

    @property
    def loop(self) -> DiagLoop:
        """Return the ``DIAg:LOOP`` command tree.

        Usage:
            - Using the ``.query()`` method will send the ``DIAg:LOOP?`` query.
            - Using the ``.verify(value)`` method will send the ``DIAg:LOOP?`` query and raise an
              AssertionError if the returned value does not match ``value``.

        Sub-properties:
            - ``.option``: The ``DIAg:LOOP:OPTion`` command.
            - ``.stop``: The ``DIAg:LOOP:STOP`` command.
        """
        return self._loop

    @property
    def mode(self) -> DiagMode:
        """Return the ``DIAg:MODe`` command.

        Description:
            - This command sets or queries the diagnostics mode.

        Usage:
            - Using the ``.query()`` method will send the ``DIAg:MODe?`` query.
            - Using the ``.verify(value)`` method will send the ``DIAg:MODe?`` query and raise an
              AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``DIAg:MODe value`` command.

        SCPI Syntax:
            ```
            - DIAg:MODe {POST|EXTENDED}
            - DIAg:MODe?
            ```

        Info:
            - ``POST`` specifies the power on self test diagnostics.
            - ``EXTENDED`` specifies the extended diagnostics.
        """
        return self._mode

    @property
    def result(self) -> DiagResult:
        """Return the ``DIAg:RESUlt`` command.

        Description:
            - This query returns both the overall diagnostics test results and the results of each
              individual test area.

        Usage:
            - Using the ``.query()`` method will send the ``DIAg:RESUlt?`` query.
            - Using the ``.verify(value)`` method will send the ``DIAg:RESUlt?`` query and raise an
              AssertionError if the returned value does not match ``value``.

        SCPI Syntax:
            ```
            - DIAg:RESUlt?
            ```

        Sub-properties:
            - ``.flag``: The ``DIAg:RESUlt:FLAg`` command.
            - ``.log``: The ``DIAg:RESUlt:LOG`` command.
        """
        return self._result

    @property
    def select(self) -> DiagSelect:
        """Return the ``DIAg:SELect`` command.

        Description:
            - This command selects an available diagnostic area.

        Usage:
            - Using the ``.write(value)`` method will send the ``DIAg:SELect value`` command.

        SCPI Syntax:
            ```
            - DIAg:SELect {ALL|IO|ANALOG|SYSTEM|ASIC|ACQ|SIGNAL|MEMORY}
            ```

        Info:
            - ``ALL`` selects all diagnostic areas.
            - ``IO`` selects the IO group.
            - ``ANALOG`` selects the ANALOG diagnostic area.
            - ``SYSTEM`` selects the SYSTEM diagnostic area.
            - ``ASIC`` selects the ASIC diagnostic area.
            - ``ACQ`` selects the ACQ diagnostic area.
            - ``SIGNAL`` selects the SIGNAL diagnostic area.
            - ``MEMORY`` selects the MEMORY diagnostic area.
        """
        return self._select

    @property
    def state(self) -> DiagState:
        """Return the ``DIAg:STATE`` command.

        Description:
            - This command starts or aborts Self Test. Abort happens after group under test
              completes.

        Usage:
            - Using the ``.write(value)`` method will send the ``DIAg:STATE value`` command.

        SCPI Syntax:
            ```
            - DIAg:STATE {EXECute|ABOrt}
            ```

        Info:
            - ``EXECute`` starts execution of the diagnostics.
            - ``ABOrt`` disables diagnostics capabilities and returns the instrument to a normal
              operating state.
        """
        return self._state



"""DMM6500 device driver module."""

from typing import Tuple

from tm_devices.commands import DMM6500Mixin
from tm_devices.driver_mixins.device_control import TSPControl
from tm_devices.driver_mixins.shared_implementations.common_tsp_error_check_mixin import (
    CommonTSPErrorCheckMixin,
)
from tm_devices.driver_mixins.shared_implementations.ieee488_2_commands import (
    LegacyTSPIEEE4882Commands,
)
from tm_devices.drivers.device import family_base_class
from tm_devices.drivers.digital_multimeters.digital_multimeter import DigitalMultimeter


@family_base_class
class DMM6500(DMM6500Mixin, CommonTSPErrorCheckMixin, TSPControl, DigitalMultimeter):
    """DMM6500 device driver."""

    _IEEE_COMMANDS_CLASS = LegacyTSPIEEE4882Commands

    ################################################################################################
    # Magic Methods
    ################################################################################################

    ################################################################################################
    # Properties
    ################################################################################################

    @property
    def ieee_cmds(self) -> LegacyTSPIEEE4882Commands:
        """Return an internal class containing methods for the standard IEEE 488.2 command set."""
        return self._ieee_cmds  # pyright: ignore[reportReturnType]

    ################################################################################################
    # Public Methods
    ################################################################################################

    ################################################################################################
    # Private Methods
    ################################################################################################
    def _get_errors(self) -> Tuple[int, Tuple[str, ...]]:
        """Get the current errors from the device.

        !!! note
            This method will clear out the error queue after reading the current errors.

        Returns:
            A tuple containing the current error code alongside a tuple of the current error
                messages.
        """
        # instrument returns exponential numbers so converting to float before int
        error_code = int(float(self.query("print(status.standard.event)")))
        err_count = int(self.query("print(eventlog.getcount(eventlog.SEV_ERROR))"))
        error_message_list = []
        if err_count:
            error_message_list = [self.query("print(eventlog.next())") for _ in range(err_count)]
        return error_code, tuple(filter(None, error_message_list))
